"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _react = _interopRequireDefault(require("react"));

var _dialog = _interopRequireDefault(require("react-toolbox/lib/dialog"));

var _PureComponent2 = _interopRequireDefault(require("./PureComponent"));

var _link = _interopRequireDefault(require("react-toolbox/lib/link"));

var _LaunchHelperModule = _interopRequireDefault(require("./LaunchHelper.module.css"));

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = (0, _getPrototypeOf2["default"])(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = (0, _getPrototypeOf2["default"])(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return (0, _possibleConstructorReturn2["default"])(this, result); }; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

var _require$default = require('./underscore_ext')["default"],
    map = _require$default.map,
    pick = _require$default.pick,
    mapObject = _require$default.mapObject,
    getIn = _require$default.getIn,
    get = _require$default.get;

var platform = require('platform');

var Rx = require('./rx')["default"];

var osFiles = {
  osxJre: {
    pattern: "ucsc_xena_macos_[_0-9]*_with_jre.dmg",
    description: "OSX installer, bundled JRE",
    help: "Recommended for OSX 10.7 and above"
  },
  win32: {
    pattern: "ucsc_xena_windows_[_0-9]*.exe",
    description: "Windows 32 bit installer, bundled JRE",
    help: "Recommended for all 32 bit versions of Windows"
  },
  win64: {
    pattern: "ucsc_xena_windows-x64_[_0-9]*.exe",
    description: "Windows 64 bit installer, bundled JRE",
    help: "Recommended for all 64 bit versions of Windows"
  },
  tar: {
    pattern: "ucsc_xena_[_0-9]*.tar.gz",
    description: "Tar archive, no updater or JRE",
    help: "Recommended for linux server deployments"
  }
},
    defaults = {
  'OS X': {
    64: 'osxJre'
  },
  'Windows': {
    32: 'win32',
    64: 'win64'
  },
  'Linux': {
    32: 'tar',
    64: 'tar'
  }
};

function findMatch(pattern, list) {
  var matches = list.filter(function (s) {
    return s.match(pattern);
  });
  return matches.length > 0 ? matches[0] : undefined;
}

var matchPaths = function matchPaths(serverFiles) {
  return mapObject(osFiles, function (obj, key) {
    return findMatch(osFiles[key].pattern, serverFiles);
  });
};

var parseInt10 = function parseInt10(s) {
  return parseInt(s, 10);
};

function osxArch() {
  var version = getIn(platform, ['os', 'version']),
      parts = version && version.match(/(\d*)\.(\d*)/).slice(1).map(parseInt10); // [maj, min, patch]

  return parts && parts[0] === 10 && parts[1] <= 6 ? 32 : 64;
}

function getOs() {
  var family = getIn(platform, ['os', 'family']);

  if (family) {
    if (family.indexOf('Windows') !== -1) {
      return 'Windows';
    }

    if (family.indexOf('OS X') !== -1) {
      return 'OS X';
    }

    if (family.indexOf('Linux') !== -1) {
      return 'Linux';
    }
  }
}

var downloadDir = 'https://genome-cancer.ucsc.edu/download/public/get-xena';
var i4jLogo = "".concat(downloadDir, "/install4j_small.png");
var updatesPath = "".concat(downloadDir, "/updates.xml");

var downloadPath = function downloadPath(name) {
  return "".concat(downloadDir, "/").concat(name);
};

var getFileName = function getFileName(e) {
  return downloadPath(e.getAttribute('fileName'));
};

var XenaDownload = /*#__PURE__*/function (_React$Component) {
  (0, _inherits2["default"])(XenaDownload, _React$Component);

  var _super = _createSuper(XenaDownload);

  function XenaDownload() {
    var _this;

    (0, _classCallCheck2["default"])(this, XenaDownload);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    (0, _defineProperty2["default"])((0, _assertThisInitialized2["default"])(_this), "state", {});
    (0, _defineProperty2["default"])((0, _assertThisInitialized2["default"])(_this), "onShowAdvanced", function () {
      _this.props.onShowAdvanced(!_this.props.advanced);
    });
    return _this;
  }

  (0, _createClass2["default"])(XenaDownload, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this2 = this;

      // Haven't been able to get the browsers to use the etags from
      // the server. It might need some other header to indicate that
      // the doc should be considered expired. This doesn't help in
      // the current deployment, because the headers for the doc have already
      // been served. A tacky work-around is to append a random query string,
      // forcing the client to bypass its cache.
      // Note that the etags do appear to work for some files, like the
      // image files.
      this.sub = Rx.Observable.ajax({
        method: 'GET',
        responseType: 'document',
        url: updatesPath + '?x=' + Math.random().toString(),
        crossDomain: true
      }).subscribe(function (xml) {
        var files = matchPaths((0, _toConsumableArray2["default"])(xml.response.getElementsByTagName('entry')).map(getFileName));

        _this2.setState({
          files: files
        }); //eslint-disable-line react/no-did-mount-set-state

      });
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.sub.unsubscribe();
    }
  }, {
    key: "render",
    value: function render() {
      var files = this.state.files,
          _this$props = this.props,
          isFirst = _this$props.isFirst,
          advanced = _this$props.advanced,
          os = getOs(),
          arch = os === 'OS X' ? osxArch() : getIn(platform, ['os', 'architecture']),
          defaultTarget = getIn(defaults, [os, arch]),
          defaultInstall = get(files, defaultTarget);
      return /*#__PURE__*/_react["default"].createElement("div", {
        className: _LaunchHelperModule["default"].download
      }, /*#__PURE__*/_react["default"].createElement("div", {
        className: _LaunchHelperModule["default"].i4j
      }, /*#__PURE__*/_react["default"].createElement("p", null, "Supported by ", /*#__PURE__*/_react["default"].createElement("a", {
        href: "https://www.ej-technologies.com/products/install4j/overview.html"
      }, /*#__PURE__*/_react["default"].createElement("img", {
        src: i4jLogo
      })))), /*#__PURE__*/_react["default"].createElement("p", null, /*#__PURE__*/_react["default"].createElement("br", null), /*#__PURE__*/_react["default"].createElement("span", {
        className: _LaunchHelperModule["default"].largeFont
      }, isFirst ? 'If this is your first time, ' : 'If nothing prompts from browser, '), defaultInstall ? /*#__PURE__*/_react["default"].createElement(_link["default"], {
        className: _LaunchHelperModule["default"].downloadLink,
        href: defaultInstall,
        label: "download & run a Local Xena hub."
      }) : 'download & run a Local Xena hub, from the list below:'), defaultInstall ? /*#__PURE__*/_react["default"].createElement("span", {
        className: _LaunchHelperModule["default"].advancedLink,
        onClick: this.onShowAdvanced
      }, advanced ? 'Fewer options...' : 'More options...') : null, defaultInstall ? /*#__PURE__*/_react["default"].createElement("br", null) : null, /*#__PURE__*/_react["default"].createElement("div", {
        className: advanced || !defaultInstall ? _LaunchHelperModule["default"].tableShow : _LaunchHelperModule["default"].table
      }, map(files ? pick(osFiles, function (_, k) {
        return files[k];
      }) : [], function (info, key) {
        return /*#__PURE__*/_react["default"].createElement(_link["default"], {
          key: key,
          className: _LaunchHelperModule["default"].downloadList,
          href: files[key],
          title: info.help,
          label: info.description
        });
      })), /*#__PURE__*/_react["default"].createElement("br", null));
    }
  }]);
  return XenaDownload;
}(_react["default"].Component);

var launchingHelp = ['Launching...', /*#__PURE__*/_react["default"].createElement("p", null, "Please click ", /*#__PURE__*/_react["default"].createElement("b", null, "Open UCSC xena"), " if you see the system dialog.")];
var statusHelp = {
  undefined: [],
  down: launchingHelp,
  started: launchingHelp,
  up: ['Your local Xena Hub is running.', /*#__PURE__*/_react["default"].createElement("p", null, "To view your data, use the \"Visualization\" button")],
  old: ['Your local Xena Hub is out of date.', /*#__PURE__*/_react["default"].createElement("p", null)],
  lost: ['We have lost contact with your Local Xena hub.', /*#__PURE__*/_react["default"].createElement("p", null, "To re-start it, you may reload this page.")]
};

var launch = function launch() {
  var i = document.getElementById('xenaLauncher');

  if (i) {
    document.body.removeChild(i);
  }

  i = document.createElement('iframe');
  i.id = 'xenaLauncher';
  document.body.appendChild(i);
  i.src = 'ucscxena://';
}; // XXX Note that we re-render on every change of the wrapped
// component. Maybe should put dialog in a separate component to
// avoid re-rendering it.


var wrap = function wrap(Comp) {
  var _class, _temp;

  return _temp = _class = /*#__PURE__*/function (_PureComponent) {
    (0, _inherits2["default"])(_class, _PureComponent);

    var _super2 = _createSuper(_class);

    function _class(props) {
      var _this3;

      (0, _classCallCheck2["default"])(this, _class);
      _this3 = _super2.call(this, props); // If we just arrived on a page that requires the hub, and status is
      // 'down' but not 'lost' (i.e. user shutdown), try to start.

      (0, _defineProperty2["default"])((0, _assertThisInitialized2["default"])(_this3), "onShowAdvanced", function (advanced) {
        _this3.setState({
          advanced: advanced
        });
      });
      (0, _defineProperty2["default"])((0, _assertThisInitialized2["default"])(_this3), "onHide", function () {
        _this3.setState({
          show: false
        });
      });
      (0, _defineProperty2["default"])((0, _assertThisInitialized2["default"])(_this3), "onShow", function () {
        _this3.setState({
          show: true
        });
      });
      (0, _defineProperty2["default"])((0, _assertThisInitialized2["default"])(_this3), "setCompRef", function (ref) {
        _this3.compRef = ref;
      });
      (0, _defineProperty2["default"])((0, _assertThisInitialized2["default"])(_this3), "actions", [{
        label: 'Help',
        target: '_blank',
        href: 'https://ucsc-xena.gitbook.io/project/local-xena-hub'
      }, {
        label: 'Close',
        onClick: _this3.onHide
      }]);
      _this3.state = {
        advanced: false,
        show: props.state.localStatus === 'down' || props.state.localStatus === 'old'
      };
      return _this3;
    }

    (0, _createClass2["default"])(_class, [{
      key: "componentDidMount",
      value: function componentDidMount() {
        if (this.props.state.localStatus === 'down') {
          launch();
        }
      }
    }, {
      key: "componentWillReceiveProps",
      value: function componentWillReceiveProps(props) {
        // If localStatus was not set (no ping yet, from server) and changes to 'down',
        // show dialog & try to launch.
        if (!this.props.state.localStatus && props.state.localStatus === 'down') {
          this.setState({
            show: true
          });
          launch();
        } else if (props.state.localStatus === 'up') {
          this.setState({
            show: false
          });
        }
      }
    }, {
      key: "render",
      value: function render() {
        var _this$state = this.state,
            advanced = _this$state.advanced,
            show = _this$state.show,
            status = this.props.state.localStatus,
            statusBadge = status === 'up' ? /*#__PURE__*/_react["default"].createElement("i", {
          title: "Connected to local Xena Hub",
          className: 'material-icons ' + _LaunchHelperModule["default"].badgeConnected,
          onClick: this.onShow
        }, "lens") : /*#__PURE__*/_react["default"].createElement("i", {
          title: "Not connected to local Xena Hub. Click for details.",
          className: 'material-icons ' + _LaunchHelperModule["default"].badgeDisconnected,
          onClick: this.onShow
        }, "lens"),
            unsupported = platform.name.match(/safari/i) && status === 'down',
            _ref = unsupported ? ["".concat(platform.name, " does not support viewing your own data. Please use Chrome or Firefox"), /*#__PURE__*/_react["default"].createElement("p", null)] : statusHelp[status],
            _ref2 = (0, _slicedToArray2["default"])(_ref, 2),
            header = _ref2[0],
            help = _ref2[1];

        return /*#__PURE__*/_react["default"].createElement(Comp, (0, _extends2["default"])({
          ref: this.setCompRef
        }, this.props, {
          badge: statusBadge
        }), /*#__PURE__*/_react["default"].createElement(_dialog["default"], {
          theme: {
            body: _LaunchHelperModule["default"].body
          },
          active: show,
          actions: this.actions,
          className: _LaunchHelperModule["default"].dialog
        }, /*#__PURE__*/_react["default"].createElement("div", {
          className: _LaunchHelperModule["default"].padding
        }), /*#__PURE__*/_react["default"].createElement("h2", {
          className: _LaunchHelperModule["default"].header
        }, header), /*#__PURE__*/_react["default"].createElement("div", {
          className: _LaunchHelperModule["default"].padding
        }), /*#__PURE__*/_react["default"].createElement("div", {
          className: _LaunchHelperModule["default"].status
        }, help, status !== 'up' && status !== 'lost' ? /*#__PURE__*/_react["default"].createElement(XenaDownload, {
          isFirst: status !== 'old',
          advanced: advanced,
          onShowAdvanced: this.onShowAdvanced
        }) : null), /*#__PURE__*/_react["default"].createElement("p", {
          className: _LaunchHelperModule["default"].footer
        }, "A Local Xena Hub is an application on your computer for loading and storing data.")));
      }
    }]);
    return _class;
  }(_PureComponent2["default"]), (0, _defineProperty2["default"])(_class, "displayName", 'LaunchHelperWrapper'), _temp;
};

var wrapLaunchHelper = function wrapLaunchHelper(shouldMount, Comp) {
  var Wrapper = wrap(Comp),
      LaunchHelperMounter = function LaunchHelperMounter(props) {
    return shouldMount(props) ? /*#__PURE__*/_react["default"].createElement(Wrapper, props) : /*#__PURE__*/_react["default"].createElement(Comp, props);
  };

  return LaunchHelperMounter;
};

var _default = wrapLaunchHelper;
exports["default"] = _default;