"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.columnsParam = columnsParam;
exports.getColumns = exports.dataset = exports.cohort = exports.getOpts = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var allParameters = require('./util')["default"].allParameters;

var _require$default = require('./underscore_ext')["default"],
    flatmap = _require$default.flatmap,
    getIn = _require$default.getIn,
    has = _require$default.has,
    identity = _require$default.identity,
    isArray = _require$default.isArray,
    isBoolean = _require$default.isBoolean,
    isObject = _require$default.isObject,
    isNumber = _require$default.isNumber,
    Let = _require$default.Let,
    map = _require$default.map,
    mapObject = _require$default.mapObject,
    merge = _require$default.merge,
    omit = _require$default.omit,
    pick = _require$default.pick,
    find = _require$default.find;

var columnOptPaths = {
  width: ['width'],
  columnLabel: ['user', 'columnLabel'],
  fieldLabel: ['user', 'fieldLabel'],
  sortDirection: ['sortDirection'],
  normalize: ['vizSettings', 'colNormalization'],
  geneAverage: ['fieldType'],
  showIntrons: ['showIntrons'],
  sortVisible: ['sortVisible']
}; // maybe should move the cleaning to models/columns.js, so we can enforce things that vary with column type.

var invalid = {}; // use reference equality to tag invalid values.

var columnOptCleaner = {
  width: function width(v) {
    return !isNumber(v) ? invalid : v < 10 ? 10 : v > 500 ? 500 : v;
  },
  sortDirection: function sortDirection(s) {
    return s === 'reverse' ? s : s === 'forward' ? s : invalid;
  },
  normalize: function normalize(v) {
    return v === 'none' ? 'none' : v === 'mean' ? 'subset' : v === 'log2' ? 'log2(x)' : v === 'normal2' ? 'normal2' : invalid;
  },
  geneAverage: function geneAverage(v) {
    return !isBoolean(v) ? invalid : v ? 'genes' : invalid;
  },
  // not so much invalid, as 'use default': 'geneProbes', or 'probes'.
  showIntrons: function showIntrons(v) {
    return isBoolean(v) ? v : invalid;
  },
  sortVisible: function sortVisible(v) {
    return isBoolean(v) ? v : invalid;
  }
};

var columnOptClean = function columnOptClean(opt, v) {
  return (columnOptCleaner[opt] || identity)(v);
};

var columnOpts = Object.keys(columnOptPaths); // invert columnOptCleaner. For creating links.

var columnOptSetter = {
  sortDirection: function sortDirection(s) {
    return s ? s : invalid;
  },
  normalize: function normalize(v) {
    return v === 'none' ? 'none' : v === 'mean' ? 'subset' : v === 'log2(x)' ? 'log2' : v === 'normal2' ? 'normal2' : invalid;
  },
  // default
  geneAverage: function geneAverage(v) {
    return v === 'genes' ? true : invalid;
  } // default

};

var columnOptSet = function columnOptSet(opt, v) {
  return (columnOptSetter[opt] || identity)(v);
};

var columnRequired = ['name', 'host', 'fields'];
var columnAllowed = [].concat(columnRequired, (0, _toConsumableArray2["default"])(columnOpts));

var mergeOpts = function mergeOpts(c) {
  return merge(omit(c, columnOpts), {
    opts: flatmap(pick(c, columnOpts), function (v, k) {
      return Let(function () {
        var cleaned = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : columnOptClean(k, v);
        return cleaned === invalid ? [] : [[columnOptPaths[k], cleaned]];
      });
    })
  });
};

var pickAllowed = function pickAllowed(c) {
  return pick(c, columnAllowed);
};

var columnSchema = function columnSchema(list) {
  return isArray(list) && list.length > 0 && list.every(function (column) {
    return isObject(column) && (has(column, 'name') && has(column, 'host') || has(column, 'cohort') && has(column, 'type')) && has(column, 'fields');
  });
};

function columnsParam() {
  var _allParameters = allParameters(),
      columns = _allParameters.columns,
      filterColumns = _allParameters.filterColumns;

  if (columns) {
    try {
      var visibleColumns = JSON.parse(columns);
      var filterOnly = filterColumns ? JSON.parse(filterColumns) : [];
      var list = visibleColumns.concat(filterOnly).map(function (c) {
        return mergeOpts(pickAllowed(c));
      });

      if (columnSchema(list)) {
        return {
          columns: list,
          visible: visibleColumns.length + 1
        }; // +1 for sampleID
      }
    } catch (e) {
      console.log("Failed to parse columns ".concat(columns));
    }
  }

  return {};
}

var getOpts = function getOpts(c) {
  return mergeOpts(pickAllowed(c)).opts;
}; // XXX not really implemented. The idea was to allow 'basic' data types
// via cohort + type.


exports.getOpts = getOpts;

var cohort = function cohort(columns) {
  return find(columns, function (c) {
    return has(c, 'cohort');
  });
}; // XXX currently unused


exports.cohort = cohort;

var dataset = function dataset(columns) {
  return columns[0].name;
};

exports.dataset = dataset;

var columnDatasetFields = function columnDatasetFields(column) {
  return _objectSpread(_objectSpread({}, JSON.parse(column.dsID)), {}, {
    fields: column.fields.join(' ')
  });
};

var columnOptions = function columnOptions(column) {
  return pick(mapObject(columnOptPaths, function (path, key) {
    return columnOptSet(key, getIn(column, path));
  }), function (v) {
    return v !== invalid;
  });
};

var encode = function encode(x) {
  return encodeURIComponent(JSON.stringify(x));
}; // extract from state


var getColumns = function getColumns(state) {
  return encode(map(omit(getIn(state, ['spreadsheet', 'columns'], {}), 'samples'), function (column) {
    return merge(columnOptions(column), columnDatasetFields(column));
  }));
};

exports.getColumns = getColumns;