"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _utils = require("./utils");

var _urlParams = require("../urlParams");

var _cohortMetaData = _interopRequireDefault(require("../cohortMetaData"));

var _query2 = _interopRequireDefault(require("./query"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var _require$default = require('../underscore_ext')["default"],
    Let = _require$default.Let,
    assocIn = _require$default.assocIn,
    dissoc = _require$default.dissoc,
    get = _require$default.get,
    identity = _require$default.identity,
    matchKeys = _require$default.matchKeys,
    pick = _require$default.pick,
    pluck = _require$default.pluck,
    uniq = _require$default.uniq,
    updateIn = _require$default.updateIn;

var _require = require('../xenaQuery'),
    cohortSummary = _require.cohortSummary,
    datasetMetadata = _require.datasetMetadata,
    datasetSamplesExamples = _require.datasetSamplesExamples,
    datasetFieldN = _require.datasetFieldN,
    datasetFieldExamples = _require.datasetFieldExamples,
    fieldCodes = _require.fieldCodes,
    datasetField = _require.datasetField,
    datasetFetch = _require.datasetFetch,
    datasetList = _require.datasetList,
    datasetSamples = _require.datasetSamples,
    sparseDataExamples = _require.sparseDataExamples,
    segmentDataExamples = _require.segmentDataExamples;

var _require2 = require('../defaultServers'),
    localHub = _require2.servers.localHub;

var _require3 = require('../xenaAdmin'),
    deleteDataset = _require3["delete"];

var _require4 = require('./common'),
    userServers = _require4.userServers;

var _require5 = require('../models/dataType'),
    ignoredType = _require5.ignoredType;

var Rx = require('../rx')["default"];

var hubsToAdd = function hubsToAdd(_ref) {
  var hubs = _ref.hubs,
      addHub = _ref.addHub;
  return (hubs || []).concat(addHub || []);
};

var hubsToRemove = function hubsToRemove(_ref2) {
  var removeHub = _ref2.removeHub;
  return removeHub || [];
};

var removeHubs = function removeHubs(state, params) {
  return hubsToRemove(params).reduce(function (state, hub) {
    return assocIn(state, ['servers', hub, 'user'], false);
  }, state);
};

var addHubs = function addHubs(state, params) {
  return hubsToAdd(params).reduce(function (state, hub) {
    return assocIn(state, ['servers', hub, 'user'], true);
  }, state);
};

var setHubs = function setHubs(state, params) {
  return addHubs(removeHubs(state, params), params);
};

var _Rx$Observable = Rx.Observable,
    ajax = _Rx$Observable.ajax,
    of = _Rx$Observable.of,
    zip = _Rx$Observable.zip,
    zipArray = _Rx$Observable.zipArray;

var ajaxGet = function ajaxGet(url) {
  return ajax({
    url: url,
    crossDomain: true,
    method: 'GET',
    responseType: 'text'
  });
};

var hostToGitURL = function hostToGitURL(host) {
  return "".concat(_cohortMetaData["default"], "/hub_").concat(host.replace(/https?:\/\//, ''), "/info.mdown");
};

var hubMeta = function hubMeta(host) {
  return ajaxGet(hostToGitURL(host))["catch"](function () {
    return ajaxGet("".concat(host, "/download/meta/info.mdown"));
  }).map(function (r) {
    return r.response;
  })["catch"](function () {
    return of({
      error: 'not available'
    });
  });
};

var cohortMeta = function cohortMeta(cohort) {
  return ajaxGet("".concat(_cohortMetaData["default"], "/cohort_").concat(cohort, "/info.mdown")).map(function (r) {
    return r.response;
  })["catch"](function () {
    return of({
      error: 'not available'
    });
  });
};

var datasetDescription = function datasetDescription(dataset) {
  return ajaxGet("".concat(_cohortMetaData["default"], "/dataset/").concat(dataset, "/info.mdown")).map(function (r) {
    return r.response;
  })["catch"](function () {
    return of({
      error: 'not available'
    });
  });
};

var getMarkDown = function getMarkDown(url) {
  return ajaxGet(url).map(function (r) {
    return r.response;
  })["catch"](function () {
    return of({
      error: 'not available'
    });
  });
}; // emit url if HEAD request succeeds


var head = function head(url) {
  return ajax({
    url: url,
    crossDomain: true,
    method: 'HEAD'
  }).map(function () {
    return url;
  });
}; // Check for dataset download link. If not there, try the link with '.gz'
// suffix. If not there, return undefined.


var checkDownload = function checkDownload(host, dataset) {
  var link = "".concat(host, "/download/").concat(dataset),
      gzlink = "".concat(link, ".gz"),
      dl = head(link),
      gzdl = head(gzlink),
      nodl = of(undefined);
  return dl["catch"](function () {
    return gzdl;
  })["catch"](function () {
    return nodl;
  });
};

var noSnippets = function noSnippets() {
  return of(undefined);
};

function fetchMatrixDataSnippets(host, dataset, meta) {
  var nProbes = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 10;
  var nSamples = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 10;
  var samplesQ = datasetSamplesExamples(host, dataset, nSamples).share(),
      fieldQ = datasetFieldExamples(host, dataset, nProbes).share(),
      codeQ = fieldQ.mergeMap(function (probes) {
    return fieldCodes(host, dataset, probes);
  }),
      dataQ = zipArray(samplesQ, fieldQ).mergeMap(function (_ref3) {
    var _ref4 = (0, _slicedToArray2["default"])(_ref3, 2),
        samples = _ref4[0],
        fields = _ref4[1];

    return datasetFetch(host, dataset, samples, fields);
  });
  return zipArray(samplesQ, fieldQ, codeQ, dataQ).map(function (_ref5) {
    var _ref6 = (0, _slicedToArray2["default"])(_ref5, 4),
        samples = _ref6[0],
        fields = _ref6[1],
        codes = _ref6[2],
        data = _ref6[3];

    return {
      samples: samples,
      fields: fields,
      codes: codes,
      data: data
    };
  })["catch"](noSnippets);
}

var mutationAttrs = function mutationAttrs(_ref7) {
  var rows = _ref7.rows;
  return _objectSpread({
    chrom: pluck(rows.position, 'chrom'),
    chromstart: pluck(rows.position, 'chromstart'),
    chromend: pluck(rows.position, 'chromend')
  }, pick(rows, 'sampleID', 'ref', 'alt', 'effect', 'amino-acid', 'gene'));
};

var fetchMutationDataSnippets = function fetchMutationDataSnippets(host, dataset) {
  var nProbes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 10;
  return sparseDataExamples(host, dataset, nProbes).map(mutationAttrs)["catch"](noSnippets);
};

var segmentAttrs = function segmentAttrs(_ref8) {
  var rows = _ref8.rows;
  return _objectSpread({
    chrom: pluck(rows.position, 'chrom'),
    chromstart: pluck(rows.position, 'chromstart'),
    chromend: pluck(rows.position, 'chromend')
  }, pick(rows, 'sampleID', 'value'));
};

var fetchSegmentedDataSnippets = function fetchSegmentedDataSnippets(host, dataset) {
  var nProbes = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 10;
  return segmentDataExamples(host, dataset, nProbes).map(segmentAttrs)["catch"](noSnippets);
};

var snippetMethod = function snippetMethod() {
  var _ref9 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref9$type = _ref9.type,
      type = _ref9$type === void 0 ? 'genomicMatrix' : _ref9$type;

  return type === 'clinicalMatrix' ? fetchMatrixDataSnippets : type === 'genomicMatrix' ? fetchMatrixDataSnippets : type === 'mutationVector' ? fetchMutationDataSnippets : type === 'genomicSegment' ? fetchSegmentedDataSnippets : noSnippets;
};

var noProbeCount = function noProbeCount() {
  return of(undefined);
};

var probeCountMethod = function probeCountMethod() {
  var _ref10 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref10$type = _ref10.type,
      type = _ref10$type === void 0 ? 'genomicMatrix' : _ref10$type;

  return type === 'clinicalMatrix' ? datasetFieldN : type === 'genomicMatrix' ? datasetFieldN : noProbeCount;
};

var datasetMetaAndLinks = function datasetMetaAndLinks(host, dataset) {
  var metaQ = datasetMetadata(host, dataset).map(function (m) {
    return m[0];
  }).share(),
      downloadQ = checkDownload(host, dataset),
      dataQ = metaQ.mergeMap(function (meta) {
    return snippetMethod(meta)(host, dataset);
  }),
      probeCountQ = metaQ.mergeMap(function (meta) {
    return probeCountMethod(meta)(host, dataset);
  }),
      probemapQ = metaQ.mergeMap(function (meta) {
    return get(meta, 'probeMap') ? checkDownload(host, meta.probeMap) : of(undefined);
  });
  return zip(metaQ, dataQ, probeCountQ, downloadQ, probemapQ, function (meta, data, probeCount, downloadLink, probemapLink) {
    return {
      meta: meta,
      data: data,
      probeCount: probeCount,
      downloadLink: downloadLink,
      probemapLink: probemapLink
    };
  });
};

var spreadsheetControls = {
  'init': function init(state, pathname, params) {
    return setHubs(state, params);
  },
  'add-host': function addHost(state, host) {
    return assocIn(state, ['servers', host], {
      user: true
    });
  },
  'remove-host': function removeHost(state, host) {
    return updateIn(state, ['servers'], function (s) {
      return dissoc(s, host);
    });
  },
  'enable-host': function enableHost(state, host, list) {
    return assocIn(state, ['servers', host, list], true);
  },
  'disable-host': function disableHost(state, host, list) {
    return assocIn(state, ['servers', host, list], false);
  }
};

var linkedHub = function linkedHub(state) {
  return state.params.host ? [state.params.host] : [];
};

var getSection = function getSection(_ref11) {
  var dataset = _ref11.dataset,
      host = _ref11.host,
      cohort = _ref11.cohort,
      allIdentifiers = _ref11.allIdentifiers,
      allSamples = _ref11.allSamples,
      markdown = _ref11.markdown;
  return markdown ? 'markdown' : allSamples ? 'samples' : allIdentifiers ? 'identifiers' : dataset && host ? 'dataset' : host ? 'hub' : cohort ? 'cohort' : 'summary';
};

var sectionDataMethods = {
  samples: function samples(_ref12) {
    var _ref12$params = _ref12.params,
        host = _ref12$params.host,
        dataset = _ref12$params.dataset;
    return [['samples', host, dataset]];
  },
  identifiers: function identifiers(_ref13) {
    var _ref13$params = _ref13.params,
        host = _ref13$params.host,
        dataset = _ref13$params.dataset;
    return [['identifiers', host, dataset]];
  },
  dataset: function dataset(_ref14) {
    var _ref14$params = _ref14.params,
        host = _ref14$params.host,
        _dataset = _ref14$params.dataset;
    return [['dataset', host, _dataset], ['datasetDescription', _dataset]];
  },
  markdown: function markdown(_ref15) {
    var _markdown = _ref15.params.markdown;
    return [['markdown', _markdown]];
  },
  cohort: function cohort(_ref16) {
    var _cohort = _ref16.params.cohort,
        spreadsheet = _ref16.spreadsheet;
    return [['cohort', _cohort]].concat((0, _toConsumableArray2["default"])(Let(function () {
      var servers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : userServers(spreadsheet);
      return servers.map(function (server) {
        return ['cohortDatasets', _cohort, server];
      });
    })));
  },
  summary: function summary(state) {
    return Let(function () {
      var servers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : uniq(userServers(state.spreadsheet).concat(linkedHub(state)));
      return servers.map(function (server) {
        return ['cohorts', server];
      });
    });
  },
  hub: function hub(_ref17) {
    var params = _ref17.params;
    return Let(function () {
      var _ref18 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _urlParams.defaultHost)(params),
          host = _ref18.host;

      return [['hubMeta', host], ['cohorts', host]];
    });
  }
};

var sectionData = function sectionData(state) {
  return state.page !== 'datapages' ? [] : Let(function () {
    var method = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : sectionDataMethods[getSection((0, _urlParams.defaultHost)(state.params))];
    return method ? method(state) : [];
  });
};

var fetchMethods = {
  samples: function samples(host, dataset) {
    return datasetSamples(host, dataset, null);
  },
  identifiers: datasetField,
  dataset: datasetMetaAndLinks,
  datasetDescription: datasetDescription,
  markdown: getMarkDown,
  cohort: cohortMeta,
  // XXX Note that this will cache lists from hubs that the user has
  // recently disabled. We only clear cache when the cohort is changed.
  // OTOH it will fetch any recently enabled hubs. So, the view should
  // iterate over the userServer list, not this cache.
  cohortDatasets: function cohortDatasets(cohort, server) {
    return datasetList(server, [cohort])["catch"](function () {
      return of([]);
    });
  },
  cohorts: function cohorts(server) {
    return cohortSummary(server, ignoredType)["catch"](function () {
      return of([]);
    });
  },
  hubMeta: hubMeta
};
var cachePolicy = {
  // cache all hosts
  cohorts: identity,
  // cache a single key: the last one updated.
  // XXX Doesn't work correctly for [host, dataset] paths, as it will accumulate
  // values until host changes. Should change this so we don't hold on to
  // multiple probe sets, for example.
  "default": function _default(state, path) {
    return updateIn(state, ['datapages', path[0]], function (item) {
      return pick(item, path[1]);
    });
  }
};

var _query = (0, _query2["default"])(fetchMethods, sectionData, cachePolicy, 'datapages'),
    fetchController = _query.controller,
    invalidatePath = _query.invalidatePath; // ['cohorts', localHub]
// ['hubMeta', localHub]
// ['samples', localHub, *]
// ['identifiers', localHub, *]
// ['dataset', localHub, *]
// ['cohortDatasets', *, localHub]


var invalidateLocalHub = Let(function () {
  var _ref19 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : matchKeys,
      any = _ref19.any;

  return function (state) {
    var datapages = get(state, 'datapages', {});
    invalidatePath(datapages, ['cohorts', localHub]);
    invalidatePath(datapages, ['hubMeta', localHub]);
    invalidatePath(datapages, ['samples', localHub, any]);
    invalidatePath(datapages, ['identifiers', localHub, any]);
    invalidatePath(datapages, ['dataset', localHub, any]);
    invalidatePath(datapages, ['cohortDatasets', any, localHub]);
  };
});

function hubChangePost(serverBus, state, newState) {
  invalidateLocalHub(newState);
}

var controls = {
  'localStatus-post!': hubChangePost,
  'localQueue-post!': hubChangePost,
  'delete-dataset-post!': function deleteDatasetPost(serverBus, state, newState, host, name) {
    return serverBus.next(['dataset-deleted', deleteDataset(host, name)]);
  }
};

var _default2 = (0, _utils.compose)(fetchController, (0, _utils.mount)((0, _utils.make)(spreadsheetControls), ['spreadsheet']), (0, _utils.make)(controls));

exports["default"] = _default2;