"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _utils = require("./utils");

var _importWorker = _interopRequireDefault(require("worker-loader!./import-worker"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var Rx = require('../rx')["default"];

var _require = require('../defaultServers'),
    servers = _require.servers;

var _require2 = require('../xenaQuery'),
    cohortSummary = _require2.cohortSummary,
    probemapList = _require2.probemapList,
    datasetStatus = _require2.datasetStatus;

var _require$default = require('../underscore_ext')["default"],
    Let = _require$default.Let,
    assoc = _require$default.assoc,
    assocIn = _require$default.assocIn,
    assocInAll = _require$default.assocInAll,
    get = _require$default.get,
    getIn = _require$default.getIn,
    has = _require$default.has,
    isArray = _require$default.isArray,
    last = _require$default.last,
    object = _require$default.object,
    pluck = _require$default.pluck,
    updateIn = _require$default.updateIn;

var loaderSocket = Rx.Observable.webSocket("ws://localhost:7222/load-events");

var watchQueueFor = function watchQueueFor(file) {
  return loaderSocket.takeWhile(function (_ref) {
    var queue = _ref.queue;
    return queue.find(function (_ref2) {
      var _ref3 = (0, _slicedToArray2["default"])(_ref2, 1),
          name = _ref3[0];

      return name === file;
    });
  }).last(null, null, 'done');
};

var worker = new _importWorker["default"](); // XXX error handling? What do we do with errors in the worker?
// For import, we collect & report, so in theory there aren't any.
// For read file, unclear.. what if there's a permissions error,
// or something? Is this possible?

var workerObs = Rx.Observable.fromEvent(worker, 'message').share();
var msgId = 0; // sendMessage wraps worker messages in ajax-like observables, by assigning
// unique ids to each request, and waiting for a single response with the
// same id. The worker must echo the id in the response.

var sendMessage = function sendMessage(msg) {
  var id = msgId++;
  worker.postMessage({
    msg: msg,
    id: id
  });
  return workerObs.filter(function (ev) {
    return ev.data.id === id;
  }).take(1).map(function (ev) {
    return ev.data.msg;
  });
};

var referenceHost = 'https://reference.xenahubs.net',
    defaultStudyName = 'My Study';

var getDefaultState = function getDefaultState() {
  return {
    wizardPage: 0,
    wizardHistory: []
  };
};

var getDefaultCustomCohort = function getDefaultCustomCohort(localCohorts) {
  var name = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultStudyName;
  var number = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;
  var _repeat = true;

  var _localCohorts, _name, _number;

  while (_repeat) {
    _repeat = false;

    if (!localCohorts.includes(name)) {
      return name;
    } else {
      _localCohorts = localCohorts;
      _name = "".concat(defaultStudyName, " (").concat(number, ")");
      _number = ++number;
      localCohorts = _localCohorts;
      name = _name;
      number = _number;
      _repeat = true;
      continue;
    }
  }
};

var importFileDone = function importFileDone(state, result) {
  return updateIn(state, ['form'], function (form) {
    return assoc(form, 'errorCheckInprogress', false, 'errors', get(result, 'errors', []), 'warnings', get(result, 'warnings', []), 'errorSnippets', get(result, 'snippets', []), 'serverError', get(result, 'serverError', undefined), 'probemapError', get(result, 'probemapError', undefined));
  });
};

var fileQueued = function fileQueued(fileName, queue) {
  return isArray(queue) && get(last(queue), 0) === fileName;
};

var readFileDone = function readFileDone(state, result) {
  return has(result, 'serverError') ? assocIn(state, ['form', 'serverError'], result.serverError) : Let(function () {
    var _ref4 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : result,
        _ref5 = (0, _slicedToArray2["default"])(_ref4, 2),
        _ref5$ = _ref5[0],
        recommended = _ref5$.recommended,
        form = _ref5$.form,
        fileContent = _ref5[1];

    return _objectSpread(_objectSpread({}, state), {}, {
      fileContent: fileContent,
      recommended: recommended,
      fileReadInProgress: false,
      form: _objectSpread(_objectSpread({}, state.form), form)
    });
  });
}; // XXX handle 'inference'


var retryFileDone = function retryFileDone(state, _ref6) {
  var _ref7 = (0, _slicedToArray2["default"])(_ref6, 2),
      _ref7$ = (0, _slicedToArray2["default"])(_ref7[0], 2),
      _ref7$$ = _ref7$[0],
      recommended = _ref7$$.recommended,
      form = _ref7$$.form,
      fileContent = _ref7$[1],
      errors = _ref7[1];

  var stateWithInference = assocInAll(state, ['recommended'], recommended, ['retryForm'], form);
  return importFileDone(assocInAll(stateWithInference, ['fileContent'], fileContent), errors);
};

var onRetryMetaData = function onRetryMetaData(state) {
  var retryForm = getIn(state, ['retryForm']) || {};
  retryForm = _objectSpread(_objectSpread({}, retryForm), {}, {
    newCohort: getDefaultCustomCohort(getIn(state, ['localCohorts']))
  });
  return assocInAll(state, ['form'], retryForm, ['retryForm'], undefined);
};

function parseServerErrors(resp) {
  var _resp = (0, _slicedToArray2["default"])(resp, 1),
      _resp$ = _resp[0],
      status = _resp$.status,
      text = _resp$.text,
      _JSON$parse = JSON.parse(text),
      error = _JSON$parse.error;

  return status === 'loaded' ? {} : {
    serverError: error || 'Unknown server error'
  };
} // emits error result in form {error, serverError, ...}


var postFile = function postFile(_ref8, ignoreWarnings) {
  var fileName = _ref8.fileName,
      form = _ref8.form,
      localProbemaps = _ref8.localProbemaps;
  return sendMessage(['postFile', fileName, form, localProbemaps, ignoreWarnings]).flatMap(function (result) {
    return fileQueued(fileName, result) ? watchQueueFor(fileName).switchMapTo(datasetStatus(servers.localHub, fileName)).map(parseServerErrors) : Rx.Observable.of(result);
  });
};

var getCohortArray = function getCohortArray(cohorts) {
  return cohorts.map(function (c) {
    return c.cohort;
  });
};

var getValueLabelList = function getValueLabelList(items) {
  return items.map(function (item) {
    return {
      label: item.label,
      value: {
        name: item.name,
        hash: item.hash
      }
    };
  });
};

var fetchLocalProbemaps = function fetchLocalProbemaps(serverBus) {
  return serverBus.next(['set-local-probemaps', probemapList(servers.localHub)]);
}; //var t0;


var importControls = {
  // XXX Important: this controller is stateful, due to maintaining a
  // web worker.
  'file': function file(state, fileHandle) {
    return assocInAll(state, ['file'], fileHandle, ['fileName'], fileHandle.name);
  },
  'import-file-post!': function importFilePost(serverBus, state, newState) {
    //		t0 = Date.now();
    serverBus.next(['import-file-done', postFile(newState)]);
  },
  // Merge in any errors & status from the file save.
  'import-file-done': importFileDone,
  // on dataset error, we may have loaded a probemap
  'import-file-done-post!': function importFileDonePost(serverBus, state, newState) {
    fetchLocalProbemaps(serverBus, state, newState); //		console.log(`load time ${(Date.now() - t0) / 1000} sec`);
  },
  'read-file': function readFile(state) {
    return assocIn(state, ['form', 'serverError'], undefined);
  },
  'read-file-post!': function readFilePost(serverBus, state, newState, fileHandle) {
    return serverBus.next(['read-file-done', sendMessage(['loadFile', newState.probemaps, fileHandle])]);
  },
  'file-read-inprogress': function fileReadInprogress(state) {
    return assocIn(state, ['fileReadInProgress'], true);
  },
  // XXX add spinner to file snippet & make sure back/forward make
  // sense while waiting on this.
  'read-file-done': readFileDone,
  'set-status': function setStatus(state, status) {
    return assocIn(state, ['status'], status);
  },
  'wizard-page': function wizardPage(state, newPage) {
    return assocIn(state, ['wizardPage'], newPage);
  },
  'wizard-page-history': function wizardPageHistory(state, newHistory) {
    return assocIn(state, ['wizardHistory'], newHistory);
  },
  'file-content': function fileContent(state, content) {
    return assocIn(state, ['fileContent'], content);
  },
  'clear-metadata': function clearMetadata(state) {
    return assocIn(state, ['form'], {});
  },
  // XXX form state is probably messed up at this point. Need a way
  // to merge in new inference when file changes.
  'retry-file-post!': function retryFilePost(serverBus, state, newState, fileHandle) {
    return serverBus.next(['retry-file-done', sendMessage(['loadFile', newState.probemaps, fileHandle]).flatMap(function (newFile) {
      return postFile(newState).map(function (errors) {
        return [newFile, errors];
      });
    })]);
  },
  'retry-file-done': retryFileDone,
  // on dataset error, we may have loaded a probemap
  'retry-file-done-post!': fetchLocalProbemaps,
  'retry-meta-data': onRetryMetaData,
  'reset-import-state': function resetImportState(state) {
    return getDefaultState(state);
  },
  'load-with-warnings-post!': function loadWithWarningsPost(serverBus, state, newState) {
    return serverBus.next(['import-file-done', postFile(newState, true)]);
  }
};
var query = {
  'get-local-cohorts-post!': function getLocalCohortsPost(serverBus) {
    return serverBus.next(['local-cohorts', cohortSummary(servers.localHub, [])]);
  },
  'local-cohorts': function localCohorts(state, cohorts) {
    var localCohorts = getCohortArray(cohorts);
    return assocInAll(state, ['localCohorts'], localCohorts, ['form', 'newCohort'], getDefaultCustomCohort(localCohorts));
  },
  'get-probemaps-post!': function getProbemapsPost(serverBus) {
    serverBus.next(['set-probemaps', probemapList(referenceHost)]);
    fetchLocalProbemaps(serverBus);
  },
  'set-probemaps': function setProbemaps(state, probemaps) {
    return assocIn(state, ['probemaps'], getValueLabelList(probemaps));
  },
  'set-local-probemaps': function setLocalProbemaps(state, probemaps) {
    return assoc(state, 'localProbemaps', object(pluck(probemaps, 'hash'), probemaps));
  }
};

var changeFormProp = function changeFormProp(propName) {
  return function (state, propValue) {
    return assocIn(state, ['form', propName], propValue);
  };
};

var formControls = {
  'file-format': changeFormProp('fileFormat'),
  'data-type': changeFormProp('dataType'),
  'cohort-radio': changeFormProp('cohortRadio'),
  'import-publicCohort': changeFormProp('publicCohort'),
  'import-localCohort': changeFormProp('localCohort'),
  'import-newCohort': changeFormProp('newCohort'),
  'errors': changeFormProp('errors'),
  'probemap': changeFormProp('probemap'),
  'assembly': changeFormProp('assembly'),
  // We can only set this 'true' from the UI. It is reset from an action handler, after import.
  'error-check-inprogress': function errorCheckInprogress(state) {
    return (// importFileDone will clear the error state if no errors are passed for the second arg.
      assocIn(importFileDone(state), ['form', 'errorCheckInprogress'], true)
    );
  }
};

var _default = (0, _utils.mount)((0, _utils.compose)((0, _utils.make)(importControls), (0, _utils.make)(query), (0, _utils.make)(formControls)), ['import']);

exports["default"] = _default;