"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = source;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _toArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toArray"));

var _utils = require("./utils");

var _require$default = require('../underscore_ext')["default"],
    assocIn = _require$default.assocIn,
    dissoc = _require$default.dissoc,
    find = _require$default.find,
    getIn = _require$default.getIn,
    identity = _require$default.identity,
    initial = _require$default.initial,
    isEqual = _require$default.isEqual,
    last = _require$default.last,
    matchKeys = _require$default.matchKeys,
    updateIn = _require$default.updateIn;

// syncs data in memory with data on server, as a function of app state.
// dataRequest is a fn of state that returns a set of paths that should be
// synced.
// fetchMethods is an object with fetch method per top-level path key.
// cachePolicy is an object with transform per top-level path key, that
// can be used to drop unused state. It is applied after merging newly
// synced data.
// mount is a top-level key under which to merge the synced data in the
// application state.
//
// returns a controller, and an imperative cache invalidation method.
function source(fetchMethods, dataRequest, cachePolicy, mount) {
  var _controls;

  // To make this more general, would need to follow the path until
  // we hit a dispatch method, then pass args.
  // XXX should this inject a common 'error' value, if the query fails?
  // We would need to update some views where we currently map to empty
  // array on error.
  var fetchData = function fetchData(_ref) {
    var _ref2 = (0, _toArray2["default"])(_ref),
        type = _ref2[0],
        args = _ref2.slice(1);

    return fetchMethods[type].apply(fetchMethods, (0, _toConsumableArray2["default"])(args));
  };

  var dispatchKey = "".concat(mount, "-merge-data"); // XXX Local mutatable state. The effects controller is stateful wrt
  // data queries.

  var queue = [];
  var outOfDate = {};

  var clearCache = function clearCache(fn) {
    return function (state, path, data) {
      return (cachePolicy[path[0]] || cachePolicy["default"])(fn(state, path, data), path);
    };
  };

  var enforceValue = function enforceValue(path, val) {
    if (val == null) {
      // Fetch methods must return a value besides null or undefined. Otherwise
      // this will create a request loop, where we fetch again because we can't
      // tell the data has already been fetched.
      console.error("Received invalid response for path ".concat(path));
      return {
        error: 'invalid value'
      };
    }

    return val;
  };

  function invalidatePath(state, pattern) {
    matchKeys(state, pattern).forEach(function (path) {
      outOfDate = assocIn(outOfDate, path, true);
    });
  }

  function validatePath(path) {
    outOfDate = updateIn(outOfDate, initial(path), function (p) {
      return p && dissoc(p, last(path));
    });
  }

  var controls = (_controls = {}, (0, _defineProperty2["default"])(_controls, dispatchKey, clearCache(function (state, path, data) {
    return assocIn(state, [mount].concat((0, _toConsumableArray2["default"])(path)), enforceValue(path, data));
  })), (0, _defineProperty2["default"])(_controls, "".concat(dispatchKey, "-post!"), function post(serverBus, state, newState, path) {
    var i = queue.findIndex(function (p) {
      return isEqual(p, path);
    });
    queue.splice(i, 1);
    validatePath(path);
  }), _controls);

  var postAction = function postAction(serverBus, state, newState) {
    var toFetch = dataRequest(newState).filter(function (path) {
      return (getIn(newState[mount], path) == null || getIn(outOfDate, path)) && !find(queue, function (p) {
        return isEqual(p, path);
      });
    });
    toFetch.forEach(function (path) {
      queue.push(path);
      serverBus.next([[dispatchKey, path], fetchData(path)]);
    });
  };

  var fetchController = {
    action: identity,
    postAction: postAction
  };
  return {
    controller: (0, _utils.compose)(fetchController, (0, _utils.make)(controls)),
    invalidatePath: invalidatePath
  };
}