"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _dataType = require("../models/dataType");

var _utils = require("./utils");

var _query2 = _interopRequireDefault(require("./query"));

var _require$default = require('../underscore_ext')["default"],
    Let = _require$default.Let,
    flatmap = _require$default.flatmap,
    fmap = _require$default.fmap,
    getIn = _require$default.getIn,
    groupBy = _require$default.groupBy,
    identity = _require$default.identity,
    map = _require$default.map,
    matchKeys = _require$default.matchKeys,
    pick = _require$default.pick,
    updateIn = _require$default.updateIn;

var xenaQuery = require('../xenaQuery');

var _require = require('../defaultServers'),
    localHub = _require.servers.localHub;

var Rx = require('../rx')["default"];

var _require2 = require('./common'),
    userServers = _require2.userServers;

var of = Rx.Observable.of; // Cohort metadata looks like
// {[cohort]: [tag, tag, ...], [cohort]: [tag, tag, ...], ...}
// We want data like
// {[tag]: [cohort, cohort, ...], [tag]: [cohort, cohort, ...], ...}

function invertCohortMeta(meta) {
  return fmap(groupBy(flatmap(meta, function (tags, cohort) {
    return tags.map(function (tag) {
      return [cohort, tag];
    });
  }), function (_ref) {
    var _ref2 = (0, _slicedToArray2["default"])(_ref, 2),
        tag = _ref2[1];

    return tag;
  }), function (cohortTags) {
    return cohortTags.map(function (_ref3) {
      var _ref4 = (0, _slicedToArray2["default"])(_ref3, 1),
          cohort = _ref4[0];

      return cohort;
    });
  });
}

var fetchCohortMeta = function fetchCohortMeta() {
  return xenaQuery.fetchCohortMeta.map(invertCohortMeta)["catch"](function () {
    return of({});
  });
};

var fetchCohortPreferred = function fetchCohortPreferred() {
  return xenaQuery.fetchCohortPreferred.map(function (cohortPreferred) {
    return fmap(cohortPreferred, function (preferred) {
      return fmap(preferred, function (_ref5) {
        var host = _ref5.host,
            dataset = _ref5.dataset;
        return JSON.stringify({
          host: host,
          name: dataset
        });
      });
    });
  })["catch"](function () {});
};

var fetchCohortPhenotype = function fetchCohortPhenotype() {
  return xenaQuery.fetchCohortPhenotype.map(function (cohortPhenotype) {
    return fmap(cohortPhenotype, function (preferred) {
      return map(preferred, function (_ref6) {
        var host = _ref6.host,
            dataset = _ref6.dataset,
            feature = _ref6.feature;
        return {
          dsID: JSON.stringify({
            host: host,
            name: dataset
          }),
          name: feature
        };
      });
    });
  });
};

var fetchCohortAnalytic = function fetchCohortAnalytic() {
  return xenaQuery.fetchCohortAnalytic;
};

var fetchTumorMap = function fetchTumorMap() {
  return xenaQuery.fetchTumorMap.map(function (res) {
    return res;
  })["catch"](function () {});
};

var phenoPat = /^phenotype/i; // match ../viwes/VariableSelect.js definition of phenotype data

var isPhenotype = function isPhenotype(ds) {
  return ds.type === 'clinicalMatrix' && (!ds.dataSubType || ds.dataSubType.match(phenoPat));
}; // {wizard: {cohortDatasets: {[cohort]: {[server]: [dataset, ...]}}}}


var allPhenoDatasets = function allPhenoDatasets(state, cohort, servers) {
  return flatmap(pick(getIn(state, ['wizard', 'cohortDatasets', state.spreadsheet.cohort.name], {}), servers), function (datasets, server) {
    return datasets.filter(isPhenotype).map(function (ds) {
      return ['cohortFeatures', cohort, server, ds.name];
    });
  });
}; // Do we really have to handle linkedHub here?? Shouldn't it be set in state?
// Depends on order of our reducers? We shouldn't have to deal with it here.


var wizardData = function wizardData(state) {
  return state.page !== 'heatmap' ? [] : Let(function () {
    var _ref7 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : state,
        spreadsheet = _ref7.spreadsheet;

    var servers = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : userServers(spreadsheet);
    return [['cohortMeta'], ['cohortPreferred'], ['cohortPhenotype'], ['cohortAnalytic'], ['cohortTumorMap']].concat((0, _toConsumableArray2["default"])(servers.map(function (server) {
      return ['serverCohorts', server];
    })), (0, _toConsumableArray2["default"])(spreadsheet.cohort ? servers.map(function (server) {
      return ['cohortDatasets', spreadsheet.cohort.name, server];
    }) : []), (0, _toConsumableArray2["default"])(spreadsheet.cohort ? allPhenoDatasets(state, spreadsheet.cohort.name, servers) : []));
  });
};

var fetchMethods = {
  // XXX Note that this will cache lists from hubs that the user has
  // recently disabled. We only clear cache when the cohort is changed.
  // OTOH it will fetch any recently enabled hubs. So, the view should
  // iterate over the userServer list, not this cache.
  cohortDatasets: function cohortDatasets(cohort, server) {
    return xenaQuery.datasetList(server, [cohort])["catch"](function () {
      return of([]);
    });
  },
  cohortFeatures: function cohortFeatures(cohort, server, dataset) {
    return xenaQuery.allFieldMetadata(server, dataset)["catch"](function () {
      return of([]);
    });
  },
  serverCohorts: function serverCohorts(server) {
    return xenaQuery.allCohorts(server, _dataType.ignoredType)["catch"](function () {
      return of([]);
    });
  },
  cohortMeta: function cohortMeta() {
    return fetchCohortMeta();
  },
  cohortPreferred: function cohortPreferred() {
    return fetchCohortPreferred();
  },
  cohortPhenotype: function cohortPhenotype() {
    return fetchCohortPhenotype();
  },
  cohortAnalytic: function cohortAnalytic() {
    return fetchCohortAnalytic();
  },
  cohortTumorMap: function cohortTumorMap() {
    return fetchTumorMap();
  }
}; // XXX review this for wizard

var cachePolicy = {
  // cache all hosts
  // ['serverCohorts', server]
  serverCohorts: identity,
  cohortMeta: identity,
  cohortPreferred: identity,
  cohortPhenotype: identity,
  cohortAnalytic: identity,
  cohortTumorMap: identity,
  // cache a single key (cohort): the last cohort updated.
  // ['cohortDatasets', cohort, server]
  // ['cohortFeatures', cohort, server, dsName]
  "default": function _default(state, path) {
    return updateIn(state, ['wizard', path[0]], function (item) {
      return pick(item, path[1]);
    });
  }
};

var _query = (0, _query2["default"])(fetchMethods, wizardData, cachePolicy, 'wizard'),
    fetchController = _query.controller,
    invalidatePath = _query.invalidatePath;

var invalidateCohorts = Let(function () {
  var _ref8 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : matchKeys,
      any = _ref8.any;

  return function (_ref9) {
    var wizard = _ref9.wizard;
    invalidatePath(wizard, ['cohortMeta']);
    invalidatePath(wizard, ['cohortPreferred']);
    invalidatePath(wizard, ['cohortPhenotype']);
    invalidatePath(wizard, ['cohortAnalytic']);
    invalidatePath(wizard, ['serverCohorts', any]);
    invalidatePath(wizard, ['cohortDatasets', any, any]);
    invalidatePath(wizard, ['cohortFeatures', any, any, any]);
  };
});
var invalidateLocalHub = Let(function () {
  var _ref10 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : matchKeys,
      any = _ref10.any;

  return function (_, __, _ref11) {
    var wizard = _ref11.wizard;
    invalidatePath(wizard, ['serverCohorts', localHub]);
    invalidatePath(wizard, ['cohortDatasets', any, localHub]);
    invalidatePath(wizard, ['cohortFeatures', any, localHub, any]);
  };
});
var controls = {
  'localStatus-post!': invalidateLocalHub,
  'localQueue-post!': invalidateLocalHub,
  'refresh-cohorts-post!': function refreshCohortsPost(serverBus, state, newState) {
    return invalidateCohorts(newState);
  }
};

var _default2 = (0, _utils.compose)(fetchController, (0, _utils.make)(controls));

exports["default"] = _default2;