"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.heatmapParam = heatmapParam;
exports.getHeatmap = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _searchParser = require("./models/searchParser");

var allParameters = require('./util')["default"].allParameters;

var _require$default = require('./underscore_ext')["default"],
    flatmap = _require$default.flatmap,
    get = _require$default.get,
    getIn = _require$default.getIn,
    groupBy = _require$default.groupBy,
    identity = _require$default.identity,
    isArray = _require$default.isArray,
    isBoolean = _require$default.isBoolean,
    isObject = _require$default.isObject,
    isString = _require$default.isString,
    Let = _require$default.Let,
    mapObject = _require$default.mapObject,
    pick = _require$default.pick;

// This is pretty much cut & paste from columnsParam, so might be
// more complex than necessary.
var heatmapOptPaths = {
  showWelcome: ['showWelcome'],
  search: ['sampleSearch'],
  searchSampleList: ['sampleSearch'],
  filter: ['sampleFilter'],
  mode: ['mode']
};

function searchIsValid(s) {
  try {
    (0, _searchParser.parse)(s);
  } catch (e) {
    return false;
  }

  return true;
}

var isStringArray = function isStringArray(a) {
  return isArray(a) && a.every(function (s) {
    return isString(s);
  });
};

var arrayToSearch = function arrayToSearch(a) {
  return a.map(function (s) {
    return "A:=\"".concat(s, "\"");
  }).join(' OR ');
};

var invalid = {}; // use reference equality to tag invalid values.

var heatmapOptCleaner = {
  mode: function mode(s) {
    return s === 'heatmap' ? s : s === 'chart' ? s : invalid;
  },
  search: function search(s) {
    return searchIsValid(s) ? s : invalid;
  },
  searchSampleList: function searchSampleList(s) {
    return isStringArray(s) ? arrayToSearch(s) : invalid;
  },
  filter: function filter(s) {
    return searchIsValid(s) ? s : invalid;
  },
  showWelcome: function showWelcome(v) {
    return isBoolean(v) ? v : invalid;
  }
};

var heatmapOptClean = function heatmapOptClean(opt, v) {
  return (heatmapOptCleaner[opt] || identity)(v);
};

var heatmapOpts = Object.keys(heatmapOptPaths);
var heatmapOptSetter = {
  mode: function mode(s) {
    return s ? s : invalid;
  },
  search: function search(s) {
    return searchIsValid(s) ? s : invalid;
  },
  searchSampleList: function searchSampleList() {
    return invalid;
  },
  // don't try to save a sample highlight
  filter: function filter() {
    return invalid;
  },
  // don't try to save a sample filter
  showWelcome: function showWelcome(v) {
    return v == null ? invalid : v;
  }
};

var heatmapOptSet = function heatmapOptSet(opt, v) {
  return (heatmapOptSetter[opt] || identity)(v);
};

var cleanOpts = function cleanOpts(c) {
  return flatmap(pick(c, heatmapOpts), function (v, k) {
    return Let(function () {
      var cleaned = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : heatmapOptClean(k, v);
      return cleaned === invalid ? [] : [[heatmapOptPaths[k], cleaned]];
    });
  });
};

var heatmapSchema = function heatmapSchema(obj) {
  return isObject(obj);
};

function heatmapParam() {
  var _allParameters = allParameters(),
      heatmap = _allParameters.heatmap;

  if (heatmap) {
    try {
      var opts = cleanOpts(JSON.parse(heatmap[0]));

      if (heatmapSchema(opts)) {
        var _groupBy = groupBy(opts, function (_ref) {
          var _ref2 = (0, _slicedToArray2["default"])(_ref, 1),
              _ref2$ = (0, _slicedToArray2["default"])(_ref2[0], 1),
              k = _ref2$[0];

          return k === 'sampleFilter';
        }),
            _groupBy$true = _groupBy["true"];

        _groupBy$true = _groupBy$true === void 0 ? [[]] : _groupBy$true;

        var _groupBy$true2 = (0, _slicedToArray2["default"])(_groupBy$true, 1),
            _groupBy$true2$ = (0, _slicedToArray2["default"])(_groupBy$true2[0], 2),
            filter = _groupBy$true2$[1],
            other = _groupBy["false"];

        return {
          heatmap: other,
          filter: filter
        };
      }

      console.log("Invalid heatmap parameter ".concat(opts));
    } catch (e) {
      console.log("Failed to parse heatmap ".concat(heatmap));
    }
  }

  return {};
}

var heatmapOptions = function heatmapOptions(heatmap) {
  return pick(mapObject(heatmapOptPaths, function (path, key) {
    return heatmapOptSet(key, getIn(heatmap, path));
  }), function (v) {
    return v !== invalid;
  });
};

var encode = function encode(x) {
  return encodeURIComponent(JSON.stringify(x));
};

var getHeatmap = function getHeatmap(state) {
  return encode(heatmapOptions(get(state, 'spreadsheet')));
};

exports.getHeatmap = getHeatmap;