"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.computeSettings = exports.typeWidth = exports.matchDatasetFields = exports.guessFields = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _multi = _interopRequireDefault(require("../multi"));

var _parsePos = _interopRequireDefault(require("../parsePos"));

var _parseInput = _interopRequireDefault(require("../parseInput"));

var _parseGeneSignature = _interopRequireDefault(require("../parseGeneSignature"));

var _heatmapColors = require("../heatmapColors");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var xenaQuery = require('../xenaQuery');

var _require$default = require('../rx')["default"],
    Observable = _require$default.Observable,
    Scheduler = _require$default.Scheduler;

var _ = require('../underscore_ext')["default"];

var _require = require('../models/fieldSpec'),
    signatureField = _require.signatureField;

// XXX duplicated in VariableSelect.
var getAssembly = function getAssembly(datasets, dsID) {
  return _.getIn(datasets, [dsID, 'assembly'], _.getIn(datasets, [dsID, 'probemapMeta', 'assembly']));
};

var guessFields = function guessFields(text, assembly) {
  var value = text.trim(),
      sig = (0, _parseGeneSignature["default"])(value),
      pos = (0, _parsePos["default"])(value, assembly),
      hasCoord = value.match(/^chr[0-9xyXY]+[pq]?:/),
      fields = sig ? sig.genes : pos ? [value] : (0, _parseInput["default"])(value);
  return {
    value: value,
    fields: fields,
    sig: sig,
    pos: pos,
    hasCoord: hasCoord
  };
};

exports.guessFields = guessFields;
var matchDatasetFields = (0, _multi["default"])(function (datasets, dsID) {
  var meta = datasets[dsID]; // Do field matching against probemap if we have one

  return meta.type === 'genomicMatrix' && meta.probemap ? 'genomicMatrix-probemap' : meta.type;
}); // XXX The error handling here isn't great, because it can leave us with a
// field of the wrong case, e.g. foxm1 vs. FOXM1, or treat a probe as a gene.
// However, it's better to handle it than to lose the observable, which wedges
// the widget. Better handling would warn the user and wait for the network
// error to clear.
// default to probes

exports.matchDatasetFields = matchDatasetFields;

matchDatasetFields.dflt = function (datasets, dsID, input) {
  var guess = guessFields(input, getAssembly(datasets, dsID)),
      fields = guess.fields,
      pos = guess.pos,
      warning = pos ? 'position-unsupported' : undefined;
  return xenaQuery.matchFields(dsID, fields).map(function (fields) {
    return _objectSpread(_objectSpread({}, guess), {}, {
      type: 'probes',
      warning: warning,
      fields: fields
    });
  })["catch"](function (err) {
    console.log(err);
    return Observable.of({
      type: 'probes',
      warning: warning,
      fields: fields
    }, Scheduler.asap);
  });
};

var geneProbeMatch = function geneProbeMatch(host, dsID, probemap, guess) {
  return Observable.zip(xenaQuery.matchGenesWithProbes(dsID, guess.fields), xenaQuery.matchFields(dsID, guess.fields), function (genes, probes) {
    return _.filter(probes).length > _.filter(genes).length ? _objectSpread(_objectSpread({}, guess), {}, {
      type: 'probes',
      fields: probes
    }) : _objectSpread(_objectSpread({}, guess), {}, {
      type: 'genes',
      fields: genes
    });
  })["catch"](function (err) {
    console.log(err);
    return Observable.of(_objectSpread(_objectSpread({}, guess), {}, {
      type: 'genes'
    }), Scheduler.asap);
  });
};

var MAX_PROBES = 500;

var chromLimit = function chromLimit(host, probemap, guess) {
  return xenaQuery.maxRange(host, probemap, guess.pos.chrom, guess.pos.baseStart, guess.pos.baseEnd, MAX_PROBES).map(function (_ref) {
    var _ref2 = (0, _slicedToArray2["default"])(_ref, 1),
        end = _ref2[0];

    return _objectSpread(_objectSpread({}, guess), {}, {
      type: 'chrom'
    }, end != null ? {
      warning: 'too-many-probes',
      start: guess.pos.baseStart,
      end: end - 1
    } : {});
  });
};

matchDatasetFields.add('genomicMatrix-probemap', function (datasets, dsID, input) {
  var guess = guessFields(input, getAssembly(datasets, dsID)),
      _JSON$parse = JSON.parse(dsID),
      host = _JSON$parse.host,
      probemap = datasets[dsID].probemap;

  return guess.pos ? chromLimit(host, probemap, guess) : geneProbeMatch(host, dsID, probemap, guess);
});

var matchAnyPosition = function matchAnyPosition(guess) {
  return Observable.of(_objectSpread(_objectSpread({}, guess), {}, {
    type: 'chrom'
  }), Scheduler.asap);
};

var normalizeGenes = function normalizeGenes(host, dsID, guess) {
  return xenaQuery.sparseDataMatchField(host, 'name2', dsID, guess.fields).map(function (fields) {
    return _objectSpread(_objectSpread({}, guess), {}, {
      type: 'genes',
      fields: fields
    });
  });
};

function matchWithAssembly(datasets, dsID, input) {
  var guess = guessFields(input, getAssembly(datasets, dsID)),
      ref = xenaQuery.refGene[datasets[dsID].assembly];
  return (guess.pos ? matchAnyPosition(guess) : normalizeGenes(ref.host, ref.name, guess))["catch"](function (err) {
    console.log(err);
    return Observable.of(_objectSpread(_objectSpread({}, guess), {}, {
      type: 'genes'
    }), Scheduler.asap);
  });
}

matchDatasetFields.add('genomicSegment', matchWithAssembly);
matchDatasetFields.add('mutationVector', matchWithAssembly);

function matchPhenotype(datasets, dsID, input) {
  return Observable.of({
    value: input,
    fields: [input]
  }, Scheduler.asap);
}

matchDatasetFields.add('clinicalMatrix', matchPhenotype); // need to handle
// phenotypic,
// null field, null dataset
// sparse,
// dense with probemap,
// dense without probemap

var typeWidth = {
  matrix: 136,
  chrom: 200
};
exports.typeWidth = typeWidth;

var typeClass = function typeClass(dataset) {
  return _.contains(['mutationVector', 'segmented'], dataset.type) ? 'chrom' : 'matrix';
}; // 'features' is a problem here, because they are not unique across datasets.
// How do we look up features w/o a dataset?


function getValueType(dataset, features, fields) {
  var type = dataset.type,
      valuetype = _.getIn(features, [fields[0], 'valuetype']);

  return type === 'mutationVector' ? 'mutation' : type === 'genomicSegment' ? 'segmented' : type === 'clinicalMatrix' && valuetype === 'category' ? 'coded' : 'float';
}

function getFieldType(dataset, fields, matches, pos) {
  var sig = matches.sig,
      type = matches.type;

  if (sig) {
    return type;
  }

  if (dataset.type === 'mutationVector') {
    return dataset.dataSubType.search(/SV|structural/i) !== -1 ? 'SV' : 'mutation';
  }

  if (dataset.type === 'genomicSegment') {
    return 'segmented';
  }

  if (dataset.type === 'clinicalMatrix') {
    return 'clinical';
  } // We treat probes in chrom view (pos) as geneProbes


  return type === 'probes' ? 'probes' : fields.length > 1 && !pos ? 'genes' : 'geneProbes';
}

function sigFields(fields, _ref3) {
  var genes = _ref3.genes,
      weights = _ref3.weights;
  var matches = new Set(fields.map(function (s) {
    return s.toLowerCase();
  }));
  return {
    missing: genes.filter(function (g) {
      return !matches.has(g.toLowerCase());
    }),
    genes: fields,
    weights: weights.filter(function (p, i) {
      return matches.has(genes[i].toLowerCase());
    })
  };
} // use vizSettings set in .json metadata


var getDefaultVizSettings = function getDefaultVizSettings(meta) {
  return _.has(meta, 'min') && _.has(meta, 'max') ? {
    vizSettings: _.pick(meta, 'min', 'max', 'minstart', 'maxstart')
  } : _.has(meta, 'origin') && _.has(meta, 'thresh') && _.has(meta, 'max') ? {
    vizSettings: _.pick(meta, 'origin', 'max', 'thresh')
  } : {};
};

var xenaField = function xenaField(datasets, features, settings) {
  return _objectSpread(_objectSpread(_objectSpread(_objectSpread({}, settings), {}, {
    fetchType: 'xena'
  }, _.getIn(settings.dataset, ['probemapMeta', 'dataSubType']) === 'regulon' ? {
    clustering: 'probes'
  } : {}), getDefaultVizSettings(settings.dataset)), {}, {
    valueType: getValueType(settings.dataset, features[settings.dsID], settings.fields),
    assembly: getAssembly(datasets, settings.dsID)
  });
};

function columnSettings(datasets, features, dsID, matches) {
  var fields = matches.fields,
      value = matches.value,
      pos = matches.pos,
      sig = matches.sig,
      dataset = datasets[dsID],
      fieldType = getFieldType(dataset, fields, matches, pos),
      _ref4 = sig ? sigFields(fields, sig) : {},
      missing = _ref4.missing,
      genes = _ref4.genes,
      weights = _ref4.weights,
      normalizedFields = sig ? [value] : pos ? ["".concat(pos.chrom, ":").concat(pos.baseStart, "-").concat(pos.baseEnd)] : _.contains(['segmented', 'mutation', 'SV'], fieldType) ? [fields[0]] : fields,
      columnLabel = !dataset.dataSubType ? dataset.label : dataset.dataSubType.match(/phenotype/i) ? '' : "".concat(dataset.dataSubType, " - ").concat(dataset.label),
      fieldLabel = sig ? "signature".concat(_.isEmpty(missing) ? '' : " (missing terms: ".concat(missing.join(', '), ")")) : dataset.type === 'clinicalMatrix' ? _.getIn(features, [dsID, fields[0], 'longtitle']) || fields[0] : normalizedFields.join(', '),
      defaults = _objectSpread(_objectSpread({}, ['geneProbes', 'segmented', 'SV'].indexOf(fieldType) !== -1 ? {
    showIntrons: true
  } : {}), {}, {
    colorClass: _heatmapColors.defaultColorClass,
    columnLabel: columnLabel,
    dataset: dataset,
    defaultNormalization: dataset.colnormalization,
    dsID: dsID,
    fields: normalizedFields,
    fieldLabel: fieldLabel,
    fieldType: fieldType,
    user: {
      columnLabel: columnLabel,
      fieldLabel: fieldLabel
    },
    value: value,
    width: typeWidth[typeClass(dataset)]
  });

  return sig ? signatureField(fieldLabel, _objectSpread(_objectSpread({}, defaults), {}, {
    signature: ['geneSignature', dsID, genes, weights],
    missing: missing
  })) : xenaField(datasets, features, defaults);
}

var computeSettings = _.curry(function (datasets, features, opts, dataset, matches) {
  var settings = columnSettings(datasets, features, dataset, matches); // XXX need a way to validate settings that depend on column type, i.e.
  // fieldType geneProbes only works for matrix with probemap.
  // Or, a possible refactor of the schema to make this simpler?

  return _.assocIn.apply(_, [settings].concat((0, _toConsumableArray2["default"])(_.flatten(opts || []))));
});

exports.computeSettings = computeSettings;