"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var heatmapColors = _interopRequireWildcard(require("../heatmapColors"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var _ = require('../underscore_ext')["default"];

var Rx = require('../rx')["default"];

var map = _.map,
    find = _.find;

var xenaQuery = require('../xenaQuery');

var widgets = require('../columnWidgets');

var _require = require('../colorScales'),
    categoryMore = _require.categoryMore;

var parsePos = require('../parsePos');

var exonLayout = require('../exonLayout');

var datasetChromProbeValues = xenaQuery.datasetChromProbeValues,
    datasetProbeValues = xenaQuery.datasetProbeValues,
    datasetGeneProbeAvg = xenaQuery.datasetGeneProbeAvg,
    datasetGeneProbesValues = xenaQuery.datasetGeneProbesValues,
    fieldCodes = xenaQuery.fieldCodes,
    refGeneRange = xenaQuery.refGeneRange;

function second(x, y) {
  return y;
}

function saveMissing(fn) {
  return function (v) {
    return v == null ? v : fn(v);
  };
}
/*
// Decide whether to normalize, perfering the user setting to the
// dataset default setting.
function shouldNormalize(vizSettings, defaultNormalization) {
	var user = _.getIn(vizSettings, ['colnormalization']);
	return user === 'subset' || (user == null && defaultNormalization === true);
}

function subbykey(subtrahend, key, val) {
	return val - subtrahend[key];
}
*/
// Returns 2d array of numbers, probes X samples.
// [[number, ...], [number, ...]]
// Performs sorting and normalization.


function computeHeatmap(vizSettings, data, fields, samples) {
  if (!data) {
    return [];
  }

  var probes = data.probes,
      values = data.values,
      transform = second; //transform = (colnormalization && mean && _.partial(subbykey, mean)) || second;

  return map(probes || fields, function (p, i) {
    var suTrans = saveMissing(function (v) {
      return transform(i, v);
    });
    return map(samples, function (s) {
      return suTrans(_.getIn(values, [i, s]));
    });
  });
} // sorted by start of probe in transcript direction (strand), for negative strand, the start of the probe is chromend
// known issue: tie break


var flopIfNegStrand = function flopIfNegStrand(strand, req) {
  var sortedReq = _.sortBy(_.zip(req.position, req.probes, req.values), strand === '-' ? function (item) {
    return -item[0].chromend;
  } : function (item) {
    return item[0].chromstart;
  }),
      _$unzip = _.unzip(sortedReq),
      _$unzip2 = (0, _slicedToArray2["default"])(_$unzip, 3),
      sortedPosition = _$unzip2[0],
      sortedProbes = _$unzip2[1],
      sortedValues = _$unzip2[2];

  return _.assoc(req, 'position', sortedPosition, 'probes', sortedProbes, 'values', sortedValues);
};

var colorCodeMap = function colorCodeMap(codes, colors) {
  return colors ? _.map(codes, function (c, i) {
    return colors[c] || categoryMore[i % categoryMore.length];
  }) : null;
};

var getCustomColor = function getCustomColor(fields, dataset) {
  return fields.length === 1 ? _.getIn(dataset, ['customcolor', fields[0]], null) : null;
};

var defaultXZoom = function defaultXZoom(pos, refGene, position, showPos, values) {
  return !showPos ? {
    start: 0,
    end: values.length
  } : pos ? {
    start: pos.baseStart,
    end: pos.baseEnd
  } : !refGene ? undefined : _.Let(function () {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : refGene,
        txStart = _ref.txStart,
        txEnd = _ref.txEnd;

    return {
      start: Math.min.apply(Math, [txStart].concat((0, _toConsumableArray2["default"])(_.pluck(position, 'chromstart')))),
      end: Math.max.apply(Math, [txEnd].concat((0, _toConsumableArray2["default"])(_.pluck(position, 'chromend'))))
    };
  });
};

var supportsClustering = function supportsClustering(_ref2) {
  var fieldType = _ref2.fieldType,
      fields = _ref2.fields;
  return _.contains(['genes', 'probes'], fieldType) && fields.length > 2 || fieldType === 'geneProbes';
};

function reOrderFields(column, data) {
  var probeOrder = _.getIn(data, ['clustering', 'probes']);

  if (supportsClustering(column) && column.clustering === 'probes' && data.status !== 'loading' && probeOrder && data.req) {
    return {
      data: _.updateIn(data, ['req', 'position'], function (position) {
        return position && probeOrder.map(function (i) {
          return position[i];
        });
      }, ['req', 'values'], function (values) {
        return values && probeOrder.map(function (i) {
          return values[i];
        });
      }, ['req', 'probes'], function (probes) {
        return probes && probeOrder.map(function (i) {
          return probes[i];
        });
      }, ['avg'], function (avg) {
        return avg && _.mapObject(avg, function (v) {
          return probeOrder.map(function (i) {
            return v[i];
          });
        });
      }),
      column: column.fieldType === 'geneProbes' ? column : _.assoc(column, 'fields', probeOrder.map(function (i) {
        return column.fields[i];
      }))
    };
  }

  return {
    column: column,
    data: data
  };
}

var reorderFieldsTransform = function reorderFieldsTransform(fn) {
  return function (column0, vizSettings, data0, samples) {
    return _.Let(function () {
      var _ref3 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : reOrderFields(column0, data0),
          column = _ref3.column,
          data = _ref3.data;

      return fn(column, vizSettings, data, samples);
    });
  };
};

var showPosition = function showPosition(column) {
  return _.getIn(column, ['dataset', 'probemapMeta', 'dataSubType']) !== 'regulon';
};

function dataToHeatmap(column, vizSettings, data, samples) {
  if (!_.get(data, 'req')) {
    return null;
  }

  var parseVizSettingCodes = function parseVizSettingCodes(str) {
    if (!_.isString(str)) {
      return undefined;
    }

    var codes = JSON.parse(str),
        observed = JSON.stringify(_.keys(codes).sort(function sortNumber(a, b) {
      return a - b;
    })),
        expected = JSON.stringify(_.range(_.keys(codes).length).map(function (x) {
      return x.toString();
    }));

    if (observed === expected) {
      return _.values(codes);
    }

    return undefined;
  };

  var req = data.req,
      avg = data.avg,
      vizSettingCodes = parseVizSettingCodes(_.getIn(column, ['vizSettings', 'codes'])),
      codes = vizSettingCodes || data.codes,
      dataset = column.dataset,
      fields = _.get(req, 'probes', column.fields),
      heatmap = computeHeatmap(vizSettings, req, fields, samples),
      customColors = colorCodeMap(codes, getCustomColor(column.fields, dataset)),
      assembly = _.getIn(dataset, ['probemapMeta', 'assembly']),
      colors = map(fields, function (p, i) {
    return heatmapColors.colorSpec(column, vizSettings, codes, {
      'values': heatmap[i],
      'mean': _.getIn(avg, ['mean', i])
    }, customColors);
  }),
      units = [_.get(dataset, 'unit')]; // column.fields is overwritten by this, which is problematic. It was
  // supposed to simplify the rendering layer by resolving whether the
  // field id maps to multiple probes, but we need the original field list
  // in the rendering layer, to determine if we support KM and gene average.
  // We could compute this in a selector, perhaps.


  return {
    fields: fields,
    fieldList: column.fields,
    heatmap: heatmap,
    avg: avg,
    assembly: assembly,
    colors: colors,
    units: units,
    codes: codes
  };
}

function geneProbesToHeatmap(column, vizSettings, data, samples) {
  var pos = parsePos(column.fields[0]);

  if (_.isEmpty(data) || _.isEmpty(data.req) || !data.refGene) {
    return null;
  }

  if (!pos && _.isEmpty(data.refGene)) {
    // got refGene, but it's empty.
    return dataToHeatmap(column, vizSettings, data, samples);
  }

  var showPos = showPosition(column),
      req = data.req,
      values = req.values,
      refGeneObj = _.first(_.values(data.refGene)),
      maxChromXZoom = defaultXZoom(pos, refGeneObj, req.position, showPos, values),
      width = column.width,
      _column$showIntrons = column.showIntrons,
      showIntrons = _column$showIntrons === void 0 ? false : _column$showIntrons,
      xzoom = column.xzoom,
      chromXZoom = showPos ? xzoom || maxChromXZoom : maxChromXZoom,
      createLayout = pos ? exonLayout.chromLayout : showIntrons ? exonLayout.intronLayout : exonLayout.layout,
      layout = createLayout(refGeneObj, width, chromXZoom, pos),
      _ref4 = xzoom || {},
      _ref4$start = _ref4.start,
      start = _ref4$start === void 0 ? 0 : _ref4$start,
      _ref4$end = _ref4.end,
      end = _ref4$end === void 0 ? req.position.length : _ref4$end,
      endIndex = end + 1,
      probesInView = showPos ? _.filterIndices(req.position, function (_ref5) {
    var chromstart = _ref5.chromstart,
        chromend = _ref5.chromend;
    return chromXZoom.start <= chromend && chromstart <= chromXZoom.end;
  }) : _.range(start, endIndex),
      dataInView = _.updateIn(data, ['req', 'position'], function (position) {
    return position ? probesInView.map(function (i) {
      return position[i];
    }) : position;
  }, ['req', 'values'], function (values) {
    return probesInView.map(function (i) {
      return values[i];
    });
  }, ['req', 'probes'], function (probes) {
    return probesInView.map(function (i) {
      return probes[i];
    });
  }, ['avg'], function (avg) {
    return _.mapObject(avg, function (v) {
      return probesInView.map(function (i) {
        return v[i];
      });
    });
  }),
      heatmapData = dataToHeatmap(column, vizSettings, dataInView, samples);

  return _objectSpread(_objectSpread({}, probesInView.length ? heatmapData : {}), {}, {
    layout: layout,
    position: dataInView.req.position,
    maxXZoom: maxChromXZoom
  });
}

function zoomableDataToHeatmap(column, vizSettings, data, samples) {
  if (!_.get(data, 'req')) {
    return null;
  }

  var req = data.req,
      values = req.values,
      _column$xzoom = column.xzoom,
      xzoom = _column$xzoom === void 0 ? {} : _column$xzoom,
      fields = column.fields,
      maxXZoomStart = 0,
      maxXZoomEnd = values.length - 1,
      _xzoom$start = xzoom.start,
      start = _xzoom$start === void 0 ? maxXZoomStart : _xzoom$start,
      _xzoom$end = xzoom.end,
      end = _xzoom$end === void 0 ? maxXZoomEnd : _xzoom$end,
      endIndex = end + 1,
      dataInView = _.updateIn(data, ['req', 'position'], function (position) {
    return position ? position.slice(start, endIndex) : position;
  }, ['req', 'values'], function (values) {
    return values.slice(start, endIndex);
  }, ['avg'], function (avg) {
    return _.mapObject(avg, function (v) {
      return v.slice(start, endIndex);
    });
  }),
      zoomedColumn = Object.assign({}, column, {
    fields: fields.slice(start, endIndex)
  }),
      heatmap = _objectSpread(_objectSpread({}, dataToHeatmap(zoomedColumn, vizSettings, dataInView, samples)), {}, {
    maxXZoom: {
      start: maxXZoomStart,
      end: maxXZoomEnd
    },
    fieldList: fields // Set field list to complete (max zoom) set of fields

  });

  return heatmap;
} //
// sort
//


var cmpNumberOrNull = _.cmpNumberOrNull;

function cmpSamples(probes, data, s1, s2) {
  var diff = data && find(data, function (f) {
    return cmpNumberOrNull(f[s1], f[s2]);
  });

  if (diff) {
    return cmpNumberOrNull(diff[s1], diff[s2]);
  } else {
    return 0;
  }
}

var cmp = function cmp(_ref6, _ref7) {
  var fields = _ref6.fields;
  var _ref7$req = _ref7.req;
  _ref7$req = _ref7$req === void 0 ? {} : _ref7$req;
  var values = _ref7$req.values,
      probes = _ref7$req.probes;
  return function (s1, s2) {
    return cmpSamples(probes || fields, values, s1, s2);
  };
}; // XXX having probes here is unfortunate.
//
// data fetches
//
// Convert nanstr and compute mean.
// XXX deprecate this & use avg selector (widgets.avg) instead.


function meanNanResponse(probes, data) {
  var values = _.map(data, function (field) {
    return _.map(field, xenaQuery.nanstr);
  }),
      mean = _.map(data, _.meannull);

  return {
    values: values,
    mean: mean
  };
}

function indexProbeGeneResponse(data) {
  var _data = (0, _slicedToArray2["default"])(data, 2),
      _data$ = _data[0],
      name = _data$.name,
      position = _data$.position,
      vals = _data[1];

  return _.extend({
    probes: name,
    position: position
  }, meanNanResponse(name, vals));
}

function fillNulls(samples, data) {
  return _.map(data, function (geneData) {
    return geneData.length === 0 ? _.times(samples.length, _.constant(null)) : geneData;
  });
}

function orderByQuery(genes, data) {
  var indx = _.invert(_.pluck(data, 'gene'));

  return _.map(genes, function (g) {
    var i = indx[g];
    return i && data[i].scores[0]; // XXX extra level of array in g.SCORES??
  });
}

function probeSpan(_ref8) {
  var position = _ref8.position;
  return position.length > 0 ? {
    chromend: _.max(_.pluck(position, 'chromend')),
    chromstart: _.min(_.pluck(position, 'chromstart')),
    strand: position[0].strand,
    chrom: position[0].chrom
  } : null;
}

function indexGeneResponse(samples, genes, data) {
  return _objectSpread({
    position: data.map(probeSpan)
  }, meanNanResponse(genes, fillNulls(samples, orderByQuery(genes, data))));
}

function indexFieldResponse(fields, resp) {
  var _resp = (0, _slicedToArray2["default"])(resp, 2),
      namePos = _resp[0],
      data = _resp[1];

  return _objectSpread({
    position: namePos && _.Let(function () {
      var _ref9 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : namePos,
          name = _ref9.name,
          position = _ref9.position;

      var posMap = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _.object(name, position);
      return fields.map(function (f) {
        return posMap[f];
      });
    })
  }, meanNanResponse(fields, data));
}

var fetch = function fetch(_ref10, samples) {
  var dsID = _ref10.dsID,
      fields = _ref10.fields;
  return datasetProbeValues(dsID, samples, fields).map(function (resp) {
    return {
      req: indexFieldResponse(fields, resp)
    };
  });
};

var fetchRefGene = function fetchRefGene(fields, assembly) {
  var _ref11 = xenaQuery.refGene[assembly] || {},
      name = _ref11.name,
      host = _ref11.host;

  return name ? xenaQuery.refGeneExons(host, name, fields) : Rx.Observable.of(null, Rx.Scheduler.asap);
};

function fetchChromProbes(_ref12, samples) {
  var dsID = _ref12.dsID,
      assembly = _ref12.assembly,
      fields = _ref12.fields;

  var _ref13 = xenaQuery.refGene[assembly] || {},
      name = _ref13.name,
      host = _ref13.host,
      pos = parsePos(fields[0]);

  return Rx.Observable.zip(refGeneRange(host, name, pos.chrom, pos.baseStart, pos.baseEnd), datasetChromProbeValues(dsID, samples, pos.chrom, pos.baseStart, pos.baseEnd), function (refGene, resp) {
    return {
      req: indexProbeGeneResponse(resp),
      refGene: refGene
    };
  });
}

var fetchGeneProbes = function fetchGeneProbes(_ref14, samples) {
  var dsID = _ref14.dsID,
      fields = _ref14.fields,
      assembly = _ref14.assembly;
  return Rx.Observable.zip(fetchRefGene(fields, assembly), datasetGeneProbesValues(dsID, samples, fields), function (refGene, resp) {
    return {
      req: flopIfNegStrand(_.getIn(refGene, [fields[0], 'strand']), indexProbeGeneResponse(resp)),
      refGene: refGene
    };
  });
};

var fetchGeneOrChromProbes = function fetchGeneOrChromProbes(field, samples) {
  return (parsePos(field.fields[0]) ? fetchChromProbes : fetchGeneProbes)(field, samples);
}; // This should really be fetchCoded. Further, it should only return a single
// code list, i.e. either a single clinical coded field, or a list of genomic
// fields all with the same code values.


var fetchFeature = function fetchFeature(_ref15, samples) {
  var dsID = _ref15.dsID,
      fields = _ref15.fields;
  return Rx.Observable.zipArray(datasetProbeValues(dsID, samples, fields).map(function (resp) {
    return indexFieldResponse(fields, resp);
  }), fieldCodes(dsID, fields)).map(function (_ref16) {
    var _ref17 = (0, _slicedToArray2["default"])(_ref16, 2),
        req = _ref17[0],
        codes = _ref17[1];

    return {
      req: req,
      codes: _.values(codes)[0]
    };
  });
};

var fetchGene = function fetchGene(_ref18, samples) {
  var dsID = _ref18.dsID,
      fields = _ref18.fields;
  return datasetGeneProbeAvg(dsID, samples, fields).map(function (resp) {
    return {
      req: indexGeneResponse(samples, fields, resp)
    };
  });
}; ////////////////////////////////
// download of on-screen data


function tsvProbeMatrix(heatmap, samples, fields, codes) {
  var fieldNames = ['sample'].concat(fields);

  var coded = _.map(fields, function (f, i) {
    return codes ? _.map(heatmap[i], _.propertyOf(codes)) : heatmap[i];
  });

  var transposed = _.zip.apply(null, coded);

  var tsvData = _.map(samples, function (sample, i) {
    return [sample].concat(transposed[i]);
  });

  return [fieldNames, tsvData];
}

function download(_ref19) {
  var column = _ref19.column,
      samples = _ref19.samples,
      sampleFormat = _ref19.sampleFormat;

  var fields = column.fields,
      heatmap = column.heatmap,
      codes = column.codes,
      tsvSamples = _.map(samples, sampleFormat);

  return tsvProbeMatrix(heatmap, tsvSamples, fields, codes);
}

function downloadCodedSampleListsJSON(_ref20) {
  var data = _ref20.data,
      samples = _ref20.samples,
      sampleFormat = _ref20.sampleFormat;

  //download json sample lists for coded column
  var values = _.getIn(data, ['req', 'values', 0]),
      codes = _.get(data, 'codes'),
      downloadData;

  if (codes.length === values.length) {
    downloadData = {
      "samples": codes
    };
  } else {
    var groupedSamples = _.groupBy(samples, function (sample) {
      return codes[values[sample]];
    });

    downloadData = _.mapObject(groupedSamples, function (val) {
      return val.map(function (sample) {
        return sampleFormat(sample);
      });
    });
  }

  return {
    type: "json",
    downloadData: downloadData
  };
}

function denseAverage(column, data) {
  var values = _.getIn(data, ['req', 'values'], []);

  return {
    avg: {
      mean: values.map(_.meannull),
      median: values.map(_.medianNull)
    }
  };
}

['probes', 'geneProbes', 'genes', 'clinical'].forEach(function (fieldType) {
  widgets.cmp.add(fieldType, cmp);
  widgets.download.add(fieldType, download);
});
['probes', 'geneProbes', 'genes', 'clinical'].forEach(function (fieldType) {
  return widgets.avg.add(fieldType, denseAverage);
});
widgets.transform.add('probes', reorderFieldsTransform(zoomableDataToHeatmap));
widgets.transform.add('geneProbes', reorderFieldsTransform(geneProbesToHeatmap));
widgets.transform.add('genes', reorderFieldsTransform(zoomableDataToHeatmap));
widgets.transform.add('clinical', reorderFieldsTransform(dataToHeatmap));
widgets.specialDownload.add('clinical', downloadCodedSampleListsJSON);
module.exports = {
  fetch: fetch,
  fetchGeneOrChromProbes: fetchGeneOrChromProbes,
  fetchGene: fetchGene,
  fetchFeature: fetchFeature
};