var _toConsumableArray = require("@babel/runtime/helpers/toConsumableArray");

var _toArray = require("@babel/runtime/helpers/toArray");

var _slicedToArray = require("@babel/runtime/helpers/slicedToArray");

var _ = require('../underscore_ext')["default"];

var _require = require('./searchParser'),
    parse = _require.parse; //var {shouldNormalize, shouldLog} = require('./denseMatrix');


var includes = function includes(target, str) {
  return str.toLowerCase().indexOf(target.toLowerCase()) !== -1;
};

function invert(matches, allSamples) {
  return _.difference(allSamples, matches);
}

function filterSampleIds(cohortSamples, cmp, str) {
  return _.filterIndices(cohortSamples, function (s) {
    return cmp(str, s);
  });
}

function searchSampleIds(cohortSamples, str) {
  return filterSampleIds(cohortSamples, includes, str);
}

function searchSampleIdsExact(cohortSamples, str) {
  return filterSampleIds(cohortSamples, function (x, y) {
    return x === y;
  }, str);
}

var searchCoded = _.curry(function (cmp, ctx, search, data) {
  var _data$req$values = _slicedToArray(data.req.values, 1),
      field = _data$req$values[0],
      codes = data.codes,
      filter = search === 'null' ? function (v) {
    return v === null;
  } : function (v) {
    return _.has(codes, v) && cmp(search, codes[v]);
  };

  return _.filterIndices(field, filter);
});

var tol = 0.01;

var near = _.curry(function (x, y) {
  return x === null || y === null ? y === x : Math.abs(x - y) < tol;
});

var searchFloat = _.curry(function (dataField, cmp, ctx, search, data) {
  var values = _.getIn(data, [dataField, 'values'], [[]]),
      searchVal = search === 'null' ? null : parseFloat(search);

  if (searchVal === null) {
    // special case for null: handle sub-columns.
    var cols = _.range(values.length),
        rows = _.range(values[0].length);

    return _.filterIndices(rows, function (i) {
      return _.every(cols, function (j) {
        return values[j][i] === null;
      });
    });
  }

  if (isNaN(searchVal) || values.length > 1) {
    // don't try to search strings against floats, and don't try to
    // search sub-columns.
    return [];
  }

  return _.filterIndices(_.map(values[0], cmp(searchVal)), _.identity);
});

var searchMutation = _.curry(function (cmp, _ref, search, data) {
  var allSamples = _ref.allSamples;
  var _data$req = data.req,
      rows = _data$req.rows,
      samplesInResp = _data$req.samplesInResp;

  if (search === 'null') {
    return invert(samplesInResp, allSamples);
  } // omit 'sample' from the variant search, because it is not stable:
  // it's an index into the sample id list, and so changes after filter.


  var matchingRows = _.filter(rows, function (row) {
    return _.any(row, function (v, k) {
      return k !== 'sample' && cmp(search, _.isString(v) ? v : String(v));
    });
  });

  return _.uniq(_.pluck(matchingRows, 'sample'));
});

var searchMethod = {
  coded: searchCoded(includes),
  "float": searchFloat('req', near),
  mutation: searchMutation(includes),
  segmented: searchFloat('avg', near),
  samples: searchSampleIds
};

var eq = _.curry(function (x, y) {
  return x === y;
});

var searchExactMethod = {
  coded: searchCoded(eq),
  "float": searchFloat('req', eq),
  mutation: searchMutation(eq),
  segmented: searchFloat('avg', eq),
  samples: searchSampleIdsExact
};

var empty = function empty() {
  return [];
};

var lt = _.curry(function (x, y) {
  return x !== null && y !== null && y < x;
});

var le = _.curry(function (x, y) {
  return x !== null && y !== null && y <= x;
});

var gt = _.curry(function (x, y) {
  return x !== null && y !== null && y > x;
});

var ge = _.curry(function (x, y) {
  return x !== null && y !== null && y >= x;
});

var searchLt = {
  coded: empty,
  mutation: empty,
  segmented: searchFloat('avg', lt),
  "float": searchFloat('req', lt)
};
var searchLe = {
  coded: empty,
  mutation: empty,
  segmented: searchFloat('avg', le),
  "float": searchFloat('req', le)
};
var searchGt = {
  coded: empty,
  mutation: empty,
  segmented: searchFloat('avg', gt),
  "float": searchFloat('req', gt)
};
var searchGe = {
  coded: empty,
  mutation: empty,
  segmented: searchFloat('avg', ge),
  "float": searchFloat('req', ge)
};

var m = function m(methods, exp, defaultMethod) {
  var _exp = _toArray(exp),
      type = _exp[0],
      args = _exp.slice(1),
      method = methods[type];

  return method ? method.apply(void 0, _toConsumableArray(args)) : defaultMethod(exp);
};

function searchAll(ctx, methods, search) {
  var cohortSamples = ctx.cohortSamples,
      columns = ctx.columns,
      data = ctx.data;
  return _.union.apply(_, _toConsumableArray(_.map(columns, function (c, key) {
    return methods[c.valueType](ctx, search, data[key]);
  })).concat([methods.samples(cohortSamples, search)]));
}

function evalFieldExp(ctx, expression, column, data) {
  if (!column || !_.get(data, 'req')) {
    return [];
  }

  return m({
    value: function value(search) {
      return searchMethod[column.valueType](ctx, search, data);
    },
    'quoted-value': function quotedValue(search) {
      return searchExactMethod[column.valueType](ctx, search, data);
    },
    ne: function ne(exp) {
      return invert(evalFieldExp(ctx, exp, column, data), ctx.allSamples);
    },
    lt: function lt(search) {
      return searchLt[column.valueType](ctx, search, data);
    },
    gt: function gt(search) {
      return searchGt[column.valueType](ctx, search, data);
    },
    le: function le(search) {
      return searchLe[column.valueType](ctx, search, data);
    },
    ge: function ge(search) {
      return searchGe[column.valueType](ctx, search, data);
    }
  }, expression);
} // XXX should rename ne to not, since we've implemented it that way.
// Unary ! or NOT can be implemented using the same operation.


function evalexp(ctx, expression) {
  return m({
    value: function value(search) {
      return searchAll(ctx, searchMethod, search);
    },
    'quoted-value': function quotedValue(search) {
      return searchAll(ctx, searchExactMethod, search);
    },
    ne: function ne(exp) {
      return invert(evalexp(ctx, exp), ctx.allSamples);
    },
    and: function and() {
      for (var _len = arguments.length, exprs = new Array(_len), _key = 0; _key < _len; _key++) {
        exprs[_key] = arguments[_key];
      }

      return _.intersection.apply(_, _toConsumableArray(exprs.map(function (e) {
        return evalexp(ctx, e);
      })));
    },
    or: function or() {
      for (var _len2 = arguments.length, exprs = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        exprs[_key2] = arguments[_key2];
      }

      return _.union.apply(_, _toConsumableArray(exprs.map(function (e) {
        return evalexp(ctx, e);
      })));
    },
    group: function group(exp) {
      return evalexp(ctx, exp);
    },
    field: function field(_field, exp) {
      return evalFieldExp(ctx, exp, ctx.columns[ctx.fieldMap[_field]], ctx.data[ctx.fieldMap[_field]]);
    }
  }, expression);
}

function treeToString(tree) {
  return m({
    cross: function cross() {
      for (var _len3 = arguments.length, exprs = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        exprs[_key3] = arguments[_key3];
      }

      return _.map(exprs, treeToString).join(' ; ');
    },
    value: function value(_value) {
      return _value;
    },
    'quoted-value': function quotedValue(value) {
      return "\"".concat(value, "\"");
    },
    and: function and() {
      for (var _len4 = arguments.length, factors = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        factors[_key4] = arguments[_key4];
      }

      return _.map(factors, treeToString).join(' ');
    },
    or: function or() {
      for (var _len5 = arguments.length, terms = new Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
        terms[_key5] = arguments[_key5];
      }

      return _.map(terms, treeToString).join(' OR ');
    },
    group: function group(exp) {
      return "(".concat(treeToString(exp), ")");
    },
    field: function field(_field2, value) {
      return "".concat(_field2, ":").concat(treeToString(value));
    },
    ne: function ne(term) {
      return "!=".concat(treeToString(term));
    },
    lt: function lt(value) {
      return "<".concat(value);
    },
    gt: function gt(value) {
      return ">".concat(value);
    },
    le: function le(value) {
      return "<=".concat(value);
    },
    ge: function ge(value) {
      return ">=".concat(value);
    }
  }, tree);
}

function evalcross(ctx, expr) {
  // do this in the parser
  var exprs = expr[0] === 'cross' ? expr.slice(1) : [expr];
  return {
    exprs: exprs.map(treeToString),
    matches: exprs.map(function (exp) {
      return evalexp(ctx, exp);
    })
  };
}

function createFieldIds(len) {
  var A = 'A'.charCodeAt(0);
  return _.times(len, function (i) {
    return String.fromCharCode(i + A);
  });
}

function createFieldMap(columnOrder) {
  return _.object(createFieldIds(columnOrder.length), columnOrder);
}

function searchSamples(search, columns, columnOrder, data, cohortSamples) {
  if (!_.get(search, 'length')) {
    return {
      exprs: null,
      matches: null
    };
  }

  var fieldMap = createFieldMap(columnOrder),
      allSamples = _.range(_.get(cohortSamples, 'length'));

  try {
    var exp = parse(search.trim());
    return evalcross({
      columns: columns,
      data: data,
      fieldMap: fieldMap,
      cohortSamples: cohortSamples,
      allSamples: allSamples
    }, exp);
  } catch (e) {
    console.log('parsing error', e);
    return {
      exprs: [],
      matches: [[]]
    };
  }
}

function remapTreeFields(tree, mapping) {
  return m({
    cross: function cross() {
      for (var _len6 = arguments.length, exprs = new Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
        exprs[_key6] = arguments[_key6];
      }

      return ['cross'].concat(_toConsumableArray(_.map(exprs, function (t) {
        return remapTreeFields(t, mapping);
      })));
    },
    and: function and() {
      for (var _len7 = arguments.length, factors = new Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
        factors[_key7] = arguments[_key7];
      }

      return ['and'].concat(_toConsumableArray(_.map(factors, function (t) {
        return remapTreeFields(t, mapping);
      })));
    },
    or: function or() {
      for (var _len8 = arguments.length, terms = new Array(_len8), _key8 = 0; _key8 < _len8; _key8++) {
        terms[_key8] = arguments[_key8];
      }

      return ['or'].concat(_toConsumableArray(_.map(terms, function (t) {
        return remapTreeFields(t, mapping);
      })));
    },
    group: function group(exp) {
      return ['group', remapTreeFields(exp, mapping)];
    },
    field: function field(_field3, value) {
      return ['field', _.get(mapping, _field3, 'XXX'), value];
    }
  }, tree, _.identity);
} // Remap field ids in search expressions. For example,
//
// oldOrder: [uuid0, uuid1, ...]
// newOrder: [uuid1, uuid0, ...]
// exp: "A:foo B:bar"
// out: "A:bar B:foo"


function remapFields(oldOrder, order, exp) {
  if (!_.get(exp, 'length')) {
    return null;
  }

  var fieldIds = createFieldIds(order.length),
      oldFieldMap = _.invert(createFieldMap(oldOrder)),
      newOrder = _.map(order, function (uuid) {
    return oldFieldMap[uuid];
  }),
      mapping = _.object(newOrder, fieldIds),
      tree = parse(exp.trim());

  return treeToString(remapTreeFields(tree, mapping));
}

module.exports = {
  searchSamples: searchSamples,
  treeToString: treeToString,
  remapFields: remapFields,
  parse: parse
};