"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _colorScales = require("./colorScales");

var _PureComponent3 = _interopRequireDefault(require("./PureComponent"));

var _reactHotLoader = require("react-hot-loader");

var _temp;

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = (0, _getPrototypeOf2["default"])(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = (0, _getPrototypeOf2["default"])(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return (0, _possibleConstructorReturn2["default"])(this, result); }; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

var _ = require('./underscore_ext')["default"];

var Rx = require('./rx')["default"];

var widgets = require('./columnWidgets');

var util = require('./util')["default"];

var Legend = require('./views/Legend');

var BandLegend = require('./views/BandLegend');

var React = require('react');

var CanvasDrawing = require('./CanvasDrawing');

var _require = require('./react-utils'),
    rxEvents = _require.rxEvents;

var _require2 = require('./drawHeatmap'),
    drawHeatmap = _require2.drawHeatmap; // Since there are multiple components in the file we have to use hot
// explicitly.


function hotOrNot(component) {
  return module.hot ? (0, _reactHotLoader.hot)(module)(component) : component;
}

var colorFns = function colorFns(vs) {
  return _.map(vs, _colorScales.colorScale);
}; //
// Tooltip
//


var prec = function () {
  var precision = 6,
      factor = Math.pow(10, precision);
  return function (val) {
    return val == null ? 'NA' : Math.round(val * factor) / factor;
  };
}(); // We're getting events with coords < 0. Not sure if this
// is a side-effect of the react event system. This will
// restrict values to the given range.


function bounded(min, max, x) {
  return x < min ? min : x > max ? max : x;
}

var posString = function posString(p) {
  return "".concat(p.chrom, ":").concat(util.addCommas(p.chromstart), "-").concat(util.addCommas(p.chromend));
}; //gb position string of the segment with 15bp extra on each side, centered at segment


var posRegionString = function posRegionString(p) {
  var pad = Math.round((p.chromend - p.chromstart) / 2);
  return "".concat(p.chrom, ":").concat(util.addCommas(p.chromstart - pad), "-").concat(util.addCommas(p.chromend + pad));
};

var gbURL = function gbURL(assembly, pos, hgtCustomtext, hubUrl) {
  var assemblyString = encodeURIComponent(assembly),
      positionString = encodeURIComponent(posString(pos)),
      regionString = encodeURIComponent(posRegionString(pos));
  return "http://genome.ucsc.edu/cgi-bin/hgTracks?db=".concat(assemblyString, "\n\t\t\t&highlight=").concat(assemblyString, ".").concat(positionString, "\n\t\t\t&position=").concat(regionString, "\n\t\t\t").concat(hubUrl ? "&hubUrl=".concat(hubUrl) : '', "\n\t\t\t").concat(hgtCustomtext ? "&hgt.customText=".concat(hgtCustomtext) : '');
};

function sigTooltip(genes, missing, val) {
  var visibleCount = 2,
      moreCount = genes.length - visibleCount,
      visible = genes.slice(0, visibleCount),
      moreLabel = moreCount > 0 ? " + ".concat(moreCount, " more") : '',
      missingLabel = _.get(missing, 'length') ? "(missing terms: ".concat(missing.join(' '), ") ") : '';
  return [['sig', // label on hover
  "signature ".concat(missingLabel, "(= ").concat(visible.join(' + ')).concat(moreLabel, ")"), // label on freeze
  "signature ".concat(missingLabel, "(= ").concat(genes.join(' + '), ")"), val]];
}

function tooltip(id, heatmap, avg, assembly, hgtCustomtext, hubUrl, fields, sampleFormat, fieldFormat, codes, position, width, zoom, samples, _ref, ev) {
  var genes = _ref.genes,
      missing = _ref.missing;

  var coord = util.eventOffset(ev),
      sampleIndex = bounded(0, samples.length, Math.floor(coord.y * zoom.count / zoom.height + zoom.index)),
      sampleID = samples[sampleIndex],
      fieldIndex = bounded(0, fields.length, Math.floor(coord.x * fields.length / width)),
      pos = _.get(position, fieldIndex),
      field = fields[fieldIndex],
      sig = !!genes;

  var val = _.getIn(heatmap, [fieldIndex, sampleIndex]),
      code = _.get(codes, val),
      label = sig ? 'signature' : fieldFormat(field);

  val = code ? code : prec(val);
  var mean = avg && prec(avg.mean[fieldIndex]),
      median = avg && prec(avg.median[fieldIndex]);
  return {
    sampleID: sampleFormat(sampleID),
    id: id,
    fieldIndex: fieldIndex,
    rows: [sig ? sigTooltip(genes, missing, val) : [['labelValue', label, val]]].concat((0, _toConsumableArray2["default"])(pos && assembly ? [[['url', "".concat(assembly, " ").concat(posString(pos)), gbURL(assembly, pos, hgtCustomtext, hubUrl)]]] : []), (0, _toConsumableArray2["default"])(!codes && mean !== 'NA' && median !== 'NA' ? [[['label', "Mean: ".concat(mean, " Median: ").concat(median)]]] : []))
  };
} //
// Legends
//


function categoryLegend(dataIn, colorScale, codes) {
  if (!colorScale) {
    return {
      colors: [],
      labels: []
    };
  } // only finds categories for the current data in the column


  var data = _.reject(_.uniq(dataIn), function (x) {
    return x == null;
  }).sort(function (v1, v2) {
    return v1 - v2;
  }),
      colors = _.map(data, colorScale),
      labels = _.map(data, function (d) {
    return codes[d];
  });

  return {
    colors: colors,
    labels: labels,
    titles: labels
  };
} // might want to use <wbr> here, instead, so cut & paste work better, but that
// will require a recursive split/flatmap to inject the <wbr> elements.


var addWordBreaks = function addWordBreaks(str) {
  return str.replace(/([_/])/g, "\u200B$1\u200B");
};

function renderFloatLegend(props) {
  var units = props.units,
      colors = props.colors,
      data = props.data,
      vizSettings = props.vizSettings;

  if (_.isEmpty(data)) {
    return null;
  }

  var colorSpec = _.max(colors, function (colorList) {
    return _.uniq(colorList.slice(Math.ceil(colorList.length / 2.0))).length;
  });

  if (colorSpec[0] === 'no-data') {
    return null;
  }

  var scale = (0, _colorScales.colorScale)(colorSpec),
      values = scale.domain(),
      footnotes = units && units[0] ? [/*#__PURE__*/React.createElement("span", {
    title: units[0]
  }, addWordBreaks(units[0]))] : null,
      hasViz = !isNaN(_.getIn(vizSettings, ['min'])),
      multiScaled = colors && colors.length > 1 && !hasViz;
  return /*#__PURE__*/React.createElement(BandLegend, {
    multiScaled: multiScaled,
    range: {
      min: _.first(values),
      max: _.last(values)
    },
    colorScale: scale,
    footnotes: footnotes,
    width: 50,
    height: 20
  });
} // Might have colorScale but no data (phenotype), no data & no colorScale,
// or data & colorScale, no colorScale &  data?


function renderCodedLegend(props) {
  var _props$data = props.data;
  _props$data = _props$data === void 0 ? [] : _props$data;

  var _props$data2 = (0, _slicedToArray2["default"])(_props$data, 1),
      data = _props$data2[0],
      codes = props.codes,
      _props$colors = props.colors,
      colors = _props$colors === void 0 ? [] : _props$colors;

  var legendProps;

  var colorfn = _.first(colorFns(colors.slice(0, 1))); // We can use domain() for categorical, but we want to filter out
  // values not in the plot. Also, we build the categorical from all
  // values in the db (even those not in the plot) so that colors will
  // match in other datasets.


  if (data && colorfn) {
    // category
    legendProps = categoryLegend(data, colorfn, codes);
  } else {
    return /*#__PURE__*/React.createElement("span", null);
  }

  return /*#__PURE__*/React.createElement(Legend, legendProps);
}

var HeatmapLegend = hotOrNot( /*#__PURE__*/function (_PureComponent) {
  (0, _inherits2["default"])(_class, _PureComponent);

  var _super = _createSuper(_class);

  function _class() {
    (0, _classCallCheck2["default"])(this, _class);
    return _super.apply(this, arguments);
  }

  (0, _createClass2["default"])(_class, [{
    key: "render",
    value: function render() {
      var column = this.props.column,
          units = column.units,
          heatmap = column.heatmap,
          colors = column.colors,
          valueType = column.valueType,
          vizSettings = column.vizSettings,
          defaultNormalization = column.defaultNormalization,
          codes = column.codes,
          props = {
        units: units,
        colors: colors,
        vizSettings: vizSettings,
        defaultNormalization: defaultNormalization,
        data: heatmap,
        coded: valueType === 'coded',
        codes: codes
      };
      return (props.coded ? renderCodedLegend : renderFloatLegend)(props);
    }
  }]);
  return _class;
}(_PureComponent3["default"])); //
// plot rendering
//

var HeatmapColumn = hotOrNot(( //
// plot rendering
//
_temp = /*#__PURE__*/function (_PureComponent2) {
  (0, _inherits2["default"])(_temp, _PureComponent2);

  var _super2 = _createSuper(_temp);

  function _temp() {
    var _this;

    (0, _classCallCheck2["default"])(this, _temp);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super2.call.apply(_super2, [this].concat(args));
    (0, _defineProperty2["default"])((0, _assertThisInitialized2["default"])(_this), "tooltip", function (ev) {
      var _this$props = _this.props,
          samples = _this$props.samples,
          data = _this$props.data,
          column = _this$props.column,
          zoom = _this$props.zoom,
          sampleFormat = _this$props.sampleFormat,
          fieldFormat = _this$props.fieldFormat,
          id = _this$props.id,
          codes = column.codes,
          avg = column.avg,
          position = column.position || _.getIn(data, ['req', 'position']),
          assembly = column.assembly,
          fields = column.fields,
          heatmap = column.heatmap,
          width = column.width,
          dataset = column.dataset,
          missing = column.missing,
          signature = column.signature,
          _ref2 = signature || [],
          _ref3 = (0, _slicedToArray2["default"])(_ref2, 3),
          genes = _ref3[2],
          hgtCustomtext = _.getIn(dataset, ['probemapMeta', 'hgt.customtext']),
          hubUrl = _.getIn(dataset, ['probemapMeta', 'huburl']);

      return tooltip(id, heatmap, avg, assembly, hgtCustomtext, hubUrl, fields, sampleFormat, fieldFormat(id), codes, position, width, zoom, samples, {
        genes: genes,
        missing: missing
      }, ev);
    });
    return _this;
  }

  (0, _createClass2["default"])(_temp, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      var _this2 = this;

      var events = rxEvents(this, 'mouseout', 'mousemove', 'mouseover'); // Compute tooltip events from mouse events.

      this.ttevents = events.mouseover.filter(function (ev) {
        return util.hasClass(ev.currentTarget, 'Tooltip-target');
      }).flatMap(function () {
        return events.mousemove.takeUntil(events.mouseout).map(function (ev) {
          return {
            data: _this2.tooltip(ev),
            open: true
          };
        }) // look up current data
        .concat(Rx.Observable.of({
          open: false
        }));
      }).subscribe(this.props.tooltip);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.ttevents.unsubscribe();
    }
  }, {
    key: "render",
    // To reduce this set of properties, we could
    //    - Drop data & move codes into the 'display' obj, outside of data
    // Might also want to copy fields into 'display', so we can drop req probes
    value: function render() {
      var _this$props2 = this.props,
          column = _this$props2.column,
          zoom = _this$props2.zoom,
          heatmap = column.heatmap,
          colors = column.colors,
          codes = column.codes;
      return /*#__PURE__*/React.createElement(CanvasDrawing, {
        ref: "plot",
        draw: drawHeatmap,
        wrapperProps: {
          className: 'Tooltip-target',
          onMouseMove: this.on.mousemove,
          onMouseOut: this.on.mouseout,
          onMouseOver: this.on.mouseover,
          onClick: this.props.onClick
        },
        codes: codes,
        width: _.get(column, 'width'),
        zoom: zoom,
        colors: colors,
        heatmapData: heatmap
      });
    }
  }]);
  return _temp;
}(_PureComponent3["default"]), _temp));

var getColumn = function getColumn(props) {
  return /*#__PURE__*/React.createElement(HeatmapColumn, props);
};

widgets.column.add("probes", getColumn);
widgets.column.add("geneProbes", getColumn);
widgets.column.add("genes", getColumn);
widgets.column.add("clinical", getColumn);

var getLegend = function getLegend(props) {
  return /*#__PURE__*/React.createElement(HeatmapLegend, props);
};

widgets.legend.add('probes', getLegend);
widgets.legend.add('geneProbes', getLegend);
widgets.legend.add('genes', getLegend);
widgets.legend.add('clinical', getLegend);