"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.findIntervals = findIntervals;
exports.RefGeneAnnotation = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _PureComponent3 = _interopRequireDefault(require("./PureComponent"));

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = (0, _getPrototypeOf2["default"])(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = (0, _getPrototypeOf2["default"])(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return (0, _possibleConstructorReturn2["default"])(this, result); }; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

var _ = require('./underscore_ext')["default"];

var React = require('react');

var ReactDOM = require('react-dom');

var Rx = require('./rx')["default"];

var intervalTree = require('static-interval-tree');

var vgcanvas = require('./vgcanvas');

var layoutPlot = require('./layoutPlot');

var matches = intervalTree.matches,
    index = intervalTree.index;
var pxTransformEach = layoutPlot.pxTransformEach;

var _require = require('./react-utils'),
    rxEvents = _require.rxEvents;

var util = require('./util')["default"];

var _require2 = require('./exonLayout'),
    chromPositionFromScreen = _require2.chromPositionFromScreen;

var _require3 = require('./colorScales'),
    isoluminant = _require3.isoluminant;

var styles = require('./refGeneExons.module.css'); // annotate an interval with cds status


var inCds = function inCds(_ref, intvl) {
  var cdsStart = _ref.cdsStart,
      cdsEnd = _ref.cdsEnd;
  return _.assoc(intvl, 'inCds', intvl.start <= cdsEnd && cdsStart <= intvl.end);
}; // split an interval at pos if it overlaps


var splitOnPos = function splitOnPos(pos, i) {
  return i.start < pos && pos <= i.end ? [_.assoc(i, 'end', pos - 1), _.assoc(i, 'start', pos)] : i;
}; // create interval record


var toIntvl = function toIntvl(start, end, i) {
  return {
    start: start,
    end: end,
    i: i
  };
}; // Create drawing intervals, by spliting exons on cds bounds, and annotating if each
// resulting region is in the cds. Each region is also annotated by its index in the
// list of exons, so we can alternate colors when rendering.
//
// findIntervals(gene :: {cdsStart :: int, cdsEnd :: int, exonStarts :: [int, ...], exonEnds :: [int, ...]})
//     :: [{start :: int, end :: int, i :: int, inCds :: boolean}, ...]


function findIntervals(gene) {
  if (_.isEmpty(gene)) {
    return [];
  }

  var cdsStart = gene.cdsStart,
      cdsEnd = gene.cdsEnd,
      exonStarts = gene.exonStarts,
      exonEnds = gene.exonEnds;
  return _.map(_.flatmap(_.flatmap(_.zip(exonStarts, exonEnds), function (_ref2, i) {
    var _ref3 = (0, _slicedToArray2["default"])(_ref2, 2),
        s = _ref3[0],
        e = _ref3[1];

    return splitOnPos(cdsStart, toIntvl(s, e, i));
  }), function (i) {
    return splitOnPos(cdsEnd + 1, i);
  }), function (i) {
    return inCds(gene, i);
  });
}

var shade1 = '#cccccc',
    //light grey
shade2 = '#000000',
    //black
shade3 = '#000080',
    //blue
shade4 = '#FF0000'; //red

function getAnnotation(index, perLaneHeight, offset) {
  return {
    utr: {
      y: offset + perLaneHeight * (index + 0.25),
      h: perLaneHeight / 2
    },
    cds: {
      y: offset + perLaneHeight * index,
      h: perLaneHeight
    }
  };
}

function drawIntroArrows(ctx, xStart, xEnd, endY, segments, strand) {
  if (xEnd - xStart < 10) {
    return;
  }

  var arrowSize = 2,
      //arrowSize
  gapSize = 4;
  ctx.strokeStyle = shade2;

  for (var i = xStart; i < xEnd; i = i + 10) {
    var found = segments.filter(function (seg) {
      return Math.abs(seg[0] - i) < gapSize || Math.abs(seg[0] - i - arrowSize) < gapSize || Math.abs(seg[1] - i) < gapSize || Math.abs(seg[1] - i - arrowSize) < gapSize;
    });

    if (_.isEmpty(found)) {
      if (strand === '+') {
        ctx.beginPath();
        ctx.moveTo(i, endY - arrowSize);
        ctx.lineTo(i + arrowSize, endY);
        ctx.stroke();
        ctx.beginPath();
        ctx.moveTo(i, endY + arrowSize);
        ctx.lineTo(i + arrowSize, endY);
        ctx.stroke();
      } else {
        // "-" strand
        ctx.beginPath();
        ctx.moveTo(i + arrowSize, endY - arrowSize);
        ctx.lineTo(i, endY);
        ctx.stroke();
        ctx.beginPath();
        ctx.moveTo(i + arrowSize, endY + arrowSize);
        ctx.lineTo(i, endY);
        ctx.stroke();
      }
    }
  }
}

var probeLayout = function probeLayout(layout, positions) {
  return layoutPlot.pxTransformFlatmap(layout, function (toPx) {
    return positions.map(function (pos) {
      return toPx([pos.chromstart, pos.chromend]);
    });
  });
};

function drawProbePositions(ctx, probePosition, height, positionHeight, width, layout) {
  var count = probePosition.length,
      screenProbes = probeLayout(layout, probePosition),
      probeHeight = 2,
      probeY = height,
      geneY = height - positionHeight,
      // conventional rainbow scale
  //		colors = _.times(count, i => `hsl(${Math.round(i * 240 / count)}, 100%, 50%)`);
  colors = _.times(count, isoluminant(0, count));

  pxTransformEach(layout, function (toPx, _ref4) {
    var _ref5 = (0, _slicedToArray2["default"])(_ref4, 2),
        start = _ref5[0],
        end = _ref5[1];

    ctx.lineWidth = 0.5;
    var positions = probePosition.map(function (p, i) {
      return [p, i];
    }).filter(function (_ref6) {
      var _ref7 = (0, _slicedToArray2["default"])(_ref6, 1),
          _ref7$ = _ref7[0],
          chromstart = _ref7$.chromstart,
          chromend = _ref7$.chromend;

      return chromstart <= end && start <= chromend;
    });
    positions.forEach(function (_ref8) {
      var _ref9 = (0, _slicedToArray2["default"])(_ref8, 2),
          _ref9$ = _ref9[0],
          chromstart = _ref9$.chromstart,
          chromend = _ref9$.chromend,
          i = _ref9[1];

      var startX = width / probePosition.length * (i + 0.5),
          middle = (chromstart + chromend) / 2,
          _toPx = toPx([middle, middle]),
          _toPx2 = (0, _slicedToArray2["default"])(_toPx, 1),
          endX = _toPx2[0];

      ctx.beginPath();
      ctx.moveTo(startX, probeY - probeHeight);
      ctx.lineTo(endX, geneY + probeHeight);
      ctx.strokeStyle = colors[i];
      ctx.stroke();
    });
  });
  screenProbes.forEach(function (_ref10, i) {
    var _ref11 = (0, _slicedToArray2["default"])(_ref10, 2),
        pxStart = _ref11[0],
        pxEnd = _ref11[1];

    ctx.beginPath();
    ctx.moveTo(pxStart, geneY);
    ctx.lineTo(pxEnd + 1, geneY);
    ctx.lineTo(pxEnd + 1, geneY + probeHeight);
    ctx.lineTo(pxStart, geneY + probeHeight);
    ctx.fillStyle = colors[i];
    ctx.fill();
  });

  _.times(screenProbes.length, function (i) {
    ctx.beginPath();
    ctx.moveTo(width / probePosition.length * i + 1, probeY - probeHeight);
    ctx.lineTo(width / probePosition.length * (i + 1) - 1, probeY - probeHeight);
    ctx.lineTo(width / probePosition.length * (i + 1) - 1, probeY);
    ctx.lineTo(width / probePosition.length * i + 1, probeY);
    ctx.fillStyle = colors[i];
    ctx.fill();
  });
} // Function to write GENE according to position


function writeGENnamepositions(ctx, xStart, xEnd, y, perLaneHeight, start, strand, label, placement) {
  var lettersize = ctx.measureText('M').width,
      labelSize = ctx.measureText(label).width,
      pad = 2;
  ctx.fillStyle = strand === "+" ? "red" : strand === "-" ? "blue" : "black";
  ctx.font = "8px"; // By default the vertical text-alignment (baseline) is set to "alphabetic" which uses the general bottom of the text for the y coordinate.

  switch (placement) {
    case "up":
      ctx.fillText(label, start, y - perLaneHeight / 2 - pad);
      break;

    case "down":
      ctx.fillText(label, start, y + perLaneHeight / 2 + lettersize + pad);
      break;

    case "right":
      ctx.fillText(label, xEnd + pad, y + lettersize / 2 - 1);
      break;

    case "left":
      ctx.fillText(label, xStart - labelSize - pad, y + lettersize / 2 - 1);
      break;
  }
}

function checkValidZone(xStart, xEnd, y, perLaneHeight, height, label, columnWidth, ctx) {
  var labelSize = ctx.measureText(label).width,
      pad = 2,
      // up down zone test line start and end
  boxStart = _.min([xStart, xEnd - labelSize + 1]),
      boxEnd = _.max([xEnd, xStart + labelSize - 1]);

  var findMaxGap = function findMaxGap(data) {
    var maxGap = 0,
        maxStart = 0,
        maxEnd = 0,
        start = -1,
        end = 0;

    for (var i = 0; i < data.length; i = i + 4) {
      if (data.slice(i, i + 3).reduce(function (a, b) {
        return a + b;
      }) === 255 * 3 || data[i + 3] === 0) {
        // white pixel
        if (start === -1) {
          // restart
          start = i;
        }

        end = i;
      } else {
        // color pixel
        if (end - start > maxGap) {
          maxGap = end - start;
          maxStart = start;
          maxEnd = end;
        }

        start = -1;
        end = -1;
      }
    }

    if (end - start > maxGap) {
      maxStart = start;
      maxEnd = end;
    }

    return [boxStart + maxStart / 4, boxStart + maxEnd / 4];
  };

  var allWhitePixel = function allWhitePixel(data) {
    for (var i = 0; i < data.length; i = i + 4) {
      if (data.slice(i, i + 3).reduce(function (a, b) {
        return a + b;
      }) !== 255 * 3 && data[i + 3] !== 0) {
        // white pixel
        return false;
      }
    }

    return true;
  };

  var checkUpZone = function checkUpZone() {
    var upBox, upStart, upEnd;

    if (y - perLaneHeight > 0) {
      upBox = ctx.getImageData(boxStart, y - perLaneHeight, boxEnd - boxStart + 1, 1);

      var _findMaxGap = findMaxGap(upBox.data);

      var _findMaxGap2 = (0, _slicedToArray2["default"])(_findMaxGap, 2);

      upStart = _findMaxGap2[0];
      upEnd = _findMaxGap2[1];

      if (labelSize < upEnd - upStart + pad) {
        return {
          confirm: true,
          placement: "up",
          start: (upEnd + upStart - labelSize) / 2
        };
      }
    }

    return;
  };

  var checkDownZone = function checkDownZone() {
    var downBox, downStart, downEnd;

    if (y + perLaneHeight < height) {
      downBox = ctx.getImageData(boxStart, y + perLaneHeight, boxEnd - boxStart + 1, 1);

      var _findMaxGap3 = findMaxGap(downBox.data);

      var _findMaxGap4 = (0, _slicedToArray2["default"])(_findMaxGap3, 2);

      downStart = _findMaxGap4[0];
      downEnd = _findMaxGap4[1];

      if (labelSize < downEnd - downStart + pad) {
        return {
          confirm: true,
          placement: "down",
          start: (downEnd + downStart - labelSize) / 2
        };
      }
    }

    return;
  };

  var checkRightZone = function checkRightZone() {
    //get the middle line the size of the GENE name located right of the Gene
    if (xEnd + pad < columnWidth) {
      var rightBox = ctx.getImageData(xEnd, y, labelSize + pad, 1);

      if (allWhitePixel(rightBox.data)) {
        return {
          confirm: true,
          placement: "right"
        };
      }
    }

    return;
  };

  var checkLeftZone = function checkLeftZone() {
    // get the middle line the size of the GENE name located left of the Gene
    if (xStart > pad) {
      var leftBox = ctx.getImageData(xStart - labelSize - pad, y, labelSize + pad, 1);

      if (allWhitePixel(leftBox.data)) {
        return {
          confirm: true,
          placement: "left"
        };
      }
    }

    return;
  };

  return checkUpZone() || checkDownZone() || checkRightZone() || checkLeftZone() || {
    confirm: false
  };
}

var RefGeneDrawing = /*#__PURE__*/function (_React$Component) {
  (0, _inherits2["default"])(RefGeneDrawing, _React$Component);

  var _super = _createSuper(RefGeneDrawing);

  function RefGeneDrawing() {
    var _this;

    (0, _classCallCheck2["default"])(this, RefGeneDrawing);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    (0, _defineProperty2["default"])((0, _assertThisInitialized2["default"])(_this), "computeAnnotationLanes", function (_ref12, single) {
      var position = _ref12.position,
          refGene = _ref12.refGene,
          height = _ref12.height,
          _ref12$positionHeight = _ref12.positionHeight,
          positionHeight = _ref12$positionHeight === void 0 ? 0 : _ref12$positionHeight;
      var bottomPad = positionHeight ? 1 : 0,
          annotationHeight = height - positionHeight - bottomPad,
          newAnnotationLanes;

      if (position && refGene) {
        var lanes = [],
            _position = (0, _slicedToArray2["default"])(position, 2),
            start = _position[0],
            end = _position[1]; //only keep genes with in the current view


        refGene = _.values(refGene).filter(function (val) {
          return val.txStart <= end && val.txEnd >= start;
        }); //multip lane no-overlapping genes

        refGene.forEach(function (val) {
          var added = lanes.some(function (lane) {
            if (lane.every(function (gene) {
              return !(val.txStart <= gene.txEnd && val.txEnd >= val.txStart);
            })) {
              return lane.push(val);
            }
          });

          if (!added) {
            // add a new lane
            if (!single || lanes.length === 0) {
              lanes.push([val]);
            } else {
              lanes[0].push(val);
            }
          }
        });

        var perLaneHeight = _.min([annotationHeight / (lanes.length || 1), 12]),
            // if no position annotation, vertically center refgene.
        // Otherwise, push it against the position annotation.
        centering = positionHeight ? 1 : 2,
            laneOffset = (annotationHeight - perLaneHeight * lanes.length) / centering;

        newAnnotationLanes = {
          arrows: !(refGene.length > 1 && single),
          lanes: lanes,
          perLaneHeight: perLaneHeight,
          laneOffset: laneOffset,
          annotationHeight: annotationHeight
        };
      } else {
        newAnnotationLanes = {
          annotationHeight: annotationHeight
        };
      } // cache for tooltip


      _this.annotationLanes = newAnnotationLanes;
    });
    (0, _defineProperty2["default"])((0, _assertThisInitialized2["default"])(_this), "draw", function (props) {
      var width = props.width,
          layout = props.layout,
          height = props.height,
          positionHeight = props.positionHeight,
          mode = props.mode,
          probePosition = props.probePosition;

      _this.computeAnnotationLanes(props, false);

      var _this$annotationLanes = _this.annotationLanes,
          lanes = _this$annotationLanes.lanes,
          perLaneHeight = _this$annotationLanes.perLaneHeight,
          arrows = _this$annotationLanes.arrows,
          laneOffset = _this$annotationLanes.laneOffset; // white background

      _this.vg.box(0, 0, width, height, 'white');

      if (!width || !layout) {
        return;
      }

      var vg = _this.vg,
          ctx = vg.context();

      if (vg.width() !== width) {
        vg.width(width);
      }

      if (_.isEmpty(layout.chrom)) {
        return;
      }

      var genePositions = []; //drawing start here, one lane at a time

      lanes.forEach(function (lane, k) {
        var annotation = getAnnotation(k, perLaneHeight, laneOffset),
            lineY = laneOffset + perLaneHeight * (k + 0.5);
        lane.forEach(function (gene) {
          var intervals = findIntervals(gene),
              indx = index(intervals),
              pGeneStart,
              pGeneEnd,
              transformedLayouts = []; //find segments for one gene

          pxTransformEach(layout, function (toPx, _ref13) {
            var _ref14 = (0, _slicedToArray2["default"])(_ref13, 2),
                start = _ref14[0],
                end = _ref14[1];

            var pLayoutStart,
                pLayoutEnd,
                nodes = matches(indx, {
              start: start,
              end: end
            }),
                segments = nodes.map(function (_ref15) {
              var i = _ref15.i,
                  start = _ref15.start,
                  end = _ref15.end,
                  inCds = _ref15.inCds;

              var _annotation = annotation[inCds ? 'cds' : 'utr'],
                  y = _annotation.y,
                  h = _annotation.h,
                  _toPx3 = toPx([start, end]),
                  _toPx4 = (0, _slicedToArray2["default"])(_toPx3, 2),
                  pstart = _toPx4[0],
                  pend = _toPx4[1],
                  shade = mode === "geneExon" ? i % 2 === 1 ? shade1 : shade2 : mode === "coordinate" ? gene.strand === '-' ? shade3 : shade4 : shade2;

              return [pstart, pend, shade, y, h];
            });

            var _toPx5 = toPx([gene.txStart, gene.txEnd]);

            var _toPx6 = (0, _slicedToArray2["default"])(_toPx5, 2);

            pLayoutStart = _toPx6[0];
            pLayoutEnd = _toPx6[1];
            // draw a line across the gene
            ctx.fillStyle = shade2;
            ctx.fillRect(pLayoutStart, lineY, pLayoutEnd - pLayoutStart, 1);

            if (arrows) {
              drawIntroArrows(ctx, pLayoutStart, pLayoutEnd, lineY, segments, mode === 'coordinate' ? gene.strand : '+');
            } // draw each segment


            _.each(segments, function (_ref16) {
              var _ref17 = (0, _slicedToArray2["default"])(_ref16, 5),
                  pstart = _ref17[0],
                  pend = _ref17[1],
                  shade = _ref17[2],
                  y = _ref17[3],
                  h = _ref17[4];

              ctx.fillStyle = shade;
              ctx.fillRect(pstart, y, pend - pstart || 1, h);
            });

            transformedLayouts.push([pLayoutStart, pLayoutEnd]);
          });
          transformedLayouts.sort(function (x, y) {
            return x[0] < y[0];
          });
          pGeneStart = transformedLayouts[0][0];
          pGeneEnd = transformedLayouts[transformedLayouts.length - 1][1];
          genePositions.push({
            pGeneStart: pGeneStart,
            pGeneEnd: pGeneEnd,
            lineY: lineY,
            label: gene.name2,
            strand: mode === 'coordinate' ? gene.strand : undefined
          });
        });
      }); //draw gene labels when relatively zoomed-in

      var labelGene = lanes.length > 0 && lanes[0].length < width / 10 ? true : false; // gene name drawing

      if (labelGene) {
        // sort to draw bigger genes' labels first
        genePositions.sort(function (x, y) {
          return y.pGeneEnd - y.pGeneStart - (x.pGeneEnd - x.pGeneStart);
        });
        var count = 0;
        genePositions.forEach(function (x) {
          // checks if is possible to write the gene name
          var _checkValidZone = checkValidZone(x.pGeneStart, x.pGeneEnd, x.lineY, perLaneHeight, perLaneHeight * lanes.length, x.label, width, ctx),
              confirm = _checkValidZone.confirm,
              placement = _checkValidZone.placement,
              start = _checkValidZone.start; // write gene name


          if (confirm) {
            count++;

            if (x.pGeneEnd - x.pGeneStart > 2 || count <= 10) {
              // draw relatively bigger genes' labels: at least 3 pixel wide or top 10 largest genes
              writeGENnamepositions(ctx, x.pGeneStart, x.pGeneEnd, x.lineY, perLaneHeight, start, x.strand, x.label, placement);
            }
          }
        });
      } // what about introns?


      if (!_.isEmpty(probePosition)) {
        drawProbePositions(ctx, probePosition, height, positionHeight, width, layout);
      }
    });
    (0, _defineProperty2["default"])((0, _assertThisInitialized2["default"])(_this), "tooltip", function (ev) {
      var _this$props = _this.props,
          layout = _this$props.layout,
          assembly = _this$props.column.assembly;

      if (!layout) {
        // gene model not loaded
        return;
      }

      var _util$eventOffset = util.eventOffset(ev),
          x = _util$eventOffset.x,
          y = _util$eventOffset.y,
          _this$annotationLanes2 = _this.annotationLanes,
          perLaneHeight = _this$annotationLanes2.perLaneHeight,
          laneOffset = _this$annotationLanes2.laneOffset,
          lanes = _this$annotationLanes2.lanes,
          rows = [],
          assemblyString = encodeURIComponent(assembly),
          contextPadding = Math.floor((layout.zoom.end - layout.zoom.start) / 4),
          posLayout = "".concat(layout.chromName, ":").concat(util.addCommas(layout.zoom.start), "-").concat(util.addCommas(layout.zoom.end)),
          posLayoutPadding = "".concat(layout.chromName, ":").concat(util.addCommas(layout.zoom.start - contextPadding), "-").concat(util.addCommas(layout.zoom.end + contextPadding)),
          posLayoutString = encodeURIComponent(posLayout),
          posLayoutPaddingString = encodeURIComponent(posLayoutPadding),
          GBurlZoom = "http://genome.ucsc.edu/cgi-bin/hgTracks?db=".concat(assemblyString, "&highlight=").concat(assemblyString, ".").concat(posLayoutString, "&position=").concat(posLayoutPaddingString);

      if (y > laneOffset && y < laneOffset + lanes.length * perLaneHeight) {
        var posStart = chromPositionFromScreen(layout, x - 0.5),
            posEnd = chromPositionFromScreen(layout, x + 0.5),
            matches = [],
            laneIndex = Math.floor((y - laneOffset) / perLaneHeight); //find which lane by y

        lanes[laneIndex].forEach(function (gene) {
          if (posEnd >= gene.txStart && posStart <= gene.txEnd) {
            matches.push(gene);
          }
        });

        if (matches.length > 0) {
          var urls = [];
          matches.forEach(function (match) {
            var posGene = "".concat(match.chrom, ":").concat(util.addCommas(match.txStart), "-").concat(util.addCommas(match.txEnd)),
                positionGeneString = encodeURIComponent(posGene),
                GBurlGene = "http://genome.ucsc.edu/cgi-bin/hgTracks?db=".concat(assemblyString, "&position=").concat(positionGeneString, "&enableHighlightingDialog=0");
            urls.push(['url', "".concat(match.name2), GBurlGene]);
          });
          rows.push([['value', 'Gene '], ['urls'].concat(urls)]);
        }
      }

      rows.push([['value', 'Column'], ['url', "".concat(assembly, " ").concat(posLayout), GBurlZoom]]);
      return {
        rows: rows
      };
    });
    return _this;
  }

  (0, _createClass2["default"])(RefGeneDrawing, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      var _this2 = this;

      var events = rxEvents(this, 'mouseout', 'mousemove', 'mouseover'); // Compute tooltip events from mouse events.

      this.ttevents = events.mouseover.filter(function (ev) {
        return util.hasClass(ev.currentTarget, 'Tooltip-target');
      }).flatMap(function () {
        return events.mousemove.takeUntil(events.mouseout).map(function (ev) {
          return {
            data: _this2.tooltip(ev),
            open: true
          };
        }) // look up current data
        .concat(Rx.Observable.of({
          open: false
        }));
      }).subscribe(this.props.tooltip);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.ttevents.unsubscribe();
    }
  }, {
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this$props2 = this.props,
          width = _this$props2.width,
          height = _this$props2.height;
      this.vg = vgcanvas(ReactDOM.findDOMNode(this.refs.canvas), width, height);
      this.draw(this.props);
    }
  }, {
    key: "shouldComponentUpdate",
    value: function shouldComponentUpdate() {
      return false;
    }
  }, {
    key: "componentWillReceiveProps",
    value: function componentWillReceiveProps(newProps) {
      if (this.vg && !_.isEqual(newProps, this.props)) {
        this.draw(newProps);
      }
    } // single: force a single lane, i.e. 'dense' mode. Unused.

  }, {
    key: "render",
    value: function render() {
      return /*#__PURE__*/React.createElement("canvas", {
        className: "Tooltip-target",
        onMouseMove: this.on.mousemove,
        onMouseOut: this.on.mouseout,
        onMouseOver: this.on.mouseover,
        onClick: this.props.onClick,
        ref: "canvas"
      });
    }
  }]);
  return RefGeneDrawing;
}(React.Component);

var RefGeneHighlight = /*#__PURE__*/function (_PureComponent) {
  (0, _inherits2["default"])(RefGeneHighlight, _PureComponent);

  var _super2 = _createSuper(RefGeneHighlight);

  function RefGeneHighlight() {
    (0, _classCallCheck2["default"])(this, RefGeneHighlight);
    return _super2.apply(this, arguments);
  }

  (0, _createClass2["default"])(RefGeneHighlight, [{
    key: "render",
    value: function render() {
      var _this$props3 = this.props,
          height = _this$props3.height,
          position = _this$props3.position,
          style = height ? {
        width: Math.max(position[1] - position[0], 1),
        height: height,
        left: position[0]
      } : {
        display: 'none'
      };
      return /*#__PURE__*/React.createElement("div", {
        className: styles.highlight
      }, /*#__PURE__*/React.createElement("div", {
        className: styles.box,
        style: style
      }));
    }
  }]);
  return RefGeneHighlight;
}(_PureComponent3["default"]);

var RefGeneAnnotation = /*#__PURE__*/function (_PureComponent2) {
  (0, _inherits2["default"])(RefGeneAnnotation, _PureComponent2);

  var _super3 = _createSuper(RefGeneAnnotation);

  function RefGeneAnnotation() {
    var _this3;

    (0, _classCallCheck2["default"])(this, RefGeneAnnotation);

    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    _this3 = _super3.call.apply(_super3, [this].concat(args));
    (0, _defineProperty2["default"])((0, _assertThisInitialized2["default"])(_this3), "state", {
      probe: undefined
    });
    return _this3;
  }

  (0, _createClass2["default"])(RefGeneAnnotation, [{
    key: "componentWillMount",
    value: function componentWillMount() {
      var _this4 = this;

      this.sub = this.props.tooltip.subscribe(function (ev) {
        if (_.getIn(ev, ['data', 'id']) === _this4.props.id) {
          _this4.setState({
            probe: _.getIn(ev, ['data', 'fieldIndex']),
            x: _.getIn(ev, ['data', 'x'])
          });
        } else if (_this4.state.probe !== null || _this4.state.x !== null) {
          _this4.setState({
            probe: undefined,
            x: undefined
          });
        }
      });
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this.sub.unsubscribe();
    }
  }, {
    key: "render",
    value: function render() {
      var _this$props4 = this.props,
          probePosition = _this$props4.probePosition,
          height = _this$props4.height,
          positionHeight = _this$props4.positionHeight,
          layout = _this$props4.layout,
          _this$state = this.state,
          probe = _this$state.probe,
          x = _this$state.x,
          highlight = probe != null && _.get(probePosition, probe) ? {
        position: probeLayout(layout, [probePosition[probe]])[0],
        height: height - positionHeight
      } : x != null ? {
        position: [x, x + 1],
        height: height - positionHeight
      } : {};
      return /*#__PURE__*/React.createElement("div", {
        className: styles.refGene
      }, /*#__PURE__*/React.createElement(RefGeneDrawing, this.props), /*#__PURE__*/React.createElement(RefGeneHighlight, highlight));
    }
  }]);
  return RefGeneAnnotation;
}(_PureComponent3["default"]);

exports.RefGeneAnnotation = RefGeneAnnotation;